@@ -26,7 +26,6 @@
 #include "core/stacktrace.h"
 #include "core/thread.h"
 #include "core/frame.h"
-#include "core/fingerprint.h"
 #include "python/stacktrace.h"
 #include "koops/stacktrace.h"
 #include "java/stacktrace.h"
@@ -48,7 +47,7 @@ file_contents(const char *directory, const char *file, char **error_message)
     char *path = sr_build_path(directory, file, NULL);
     char *contents = sr_file_to_string(path, error_message);
 
-    free(path);
+    g_free(path);
     return contents;
 }
 
@@ -65,7 +64,7 @@ sr_abrt_print_report_from_dir(const char *directory,
     char *json = sr_report_to_json(report);
     sr_report_free(report);
     puts(json);
-    free(json);
+    g_free(json);
     return true;
 }
 
@@ -81,7 +80,7 @@ fulfill_missing_values(struct sr_core_stacktrace *core_stacktrace)
             if (!frame->file_name && frame->function_name
                 && strcmp("__kernel_vsyscall", frame->function_name) == 0)
             {
-                frame->file_name = sr_strdup("kernel");
+                frame->file_name = g_strdup("kernel");
             }
             frame = frame->next;
         }
@@ -91,7 +90,7 @@ fulfill_missing_values(struct sr_core_stacktrace *core_stacktrace)
 
 static bool
 create_core_stacktrace(const char *directory, const char *gdb_output,
-                       bool hash_fingerprints, char **error_message)
+                       bool unused, char **error_message)
 {
     char *executable_contents = file_contents(directory, "executable",
                                               error_message);
@@ -109,26 +108,18 @@ create_core_stacktrace(const char *directory, const char *gdb_output,
         core_stacktrace = sr_parse_coredump(coredump_filename,
                 executable_contents, error_message);
 
-    free(executable_contents);
-    free(coredump_filename);
+    g_free(executable_contents);
+    g_free(coredump_filename);
     if (!core_stacktrace)
         return false;
 
     fulfill_missing_values(core_stacktrace);
 
-#if 0
-    sr_core_fingerprint_generate(core_stacktrace,
-                                 error_message);
-
-    if (hash_fingerprints)
-        sr_core_fingerprint_hash(core_stacktrace);
-#endif
-
     char *json = sr_core_stacktrace_to_json(core_stacktrace);
 
     // Add newline to the end of core stacktrace file to make text
     // editors happy.
-    json = sr_realloc(json, strlen(json) + 2);
+    json = g_realloc(json, strlen(json) + 2);
     strcat(json, "\n");
 
     char *core_backtrace_filename = sr_build_path(directory, "core_backtrace", NULL);
@@ -136,8 +127,8 @@ create_core_stacktrace(const char *directory, const char *gdb_output,
                                     json,
                                     error_message);
 
-    free(core_backtrace_filename);
-    free(json);
+    g_free(core_backtrace_filename);
+    g_free(json);
     sr_core_stacktrace_free(core_stacktrace);
     return success;
 }
@@ -145,7 +136,7 @@ create_core_stacktrace(const char *directory, const char *gdb_output,
 bool
 sr_abrt_create_core_stacktrace_from_gdb(const char *directory,
                                         const char *gdb_output,
-                                        bool hash_fingerprints,
+                                        bool unused,
                                         char **error_message)
 {
     if (!gdb_output)
@@ -154,17 +145,15 @@ sr_abrt_create_core_stacktrace_from_gdb(const char *directory,
         return false;
     }
 
-    return create_core_stacktrace(directory, gdb_output, hash_fingerprints,
-                                  error_message);
+    return create_core_stacktrace(directory, gdb_output, unused, error_message);
 }
 
 bool
 sr_abrt_create_core_stacktrace(const char *directory,
-                               bool hash_fingerprints,
+                               bool unused,
                                char **error_message)
 {
-    return create_core_stacktrace(directory, NULL, hash_fingerprints,
-                                  error_message);
+    return create_core_stacktrace(directory, NULL, unused, error_message);
 }
 
 struct sr_core_stracetrace_unwind_state *
@@ -194,7 +183,7 @@ sr_abrt_get_core_stacktrace_from_core_hook_generate(pid_t thread_id,
 
     // Add newline to the end of core stacktrace file to make text
     // editors happy.
-    json = sr_realloc(json, strlen(json) + 2);
+    json = g_realloc(json, strlen(json) + 2);
     strcat(json, "\n");
 
     return json;
@@ -237,7 +226,7 @@ sr_abrt_parse_dso_list(const char *text)
         }
 
         // Parse the package NEVRA.
-        char *nevra = sr_strndup(pos, end - pos);
+        char *nevra = g_strndup(pos, end - pos);
         struct sr_rpm_package *dso_package = sr_rpm_package_new();
         bool success = sr_rpm_package_parse_nevra(nevra,
                                                   &dso_package->name,
@@ -246,7 +235,7 @@ sr_abrt_parse_dso_list(const char *text)
                                                   &dso_package->release,
                                                   &dso_package->architecture);
 
-        free(nevra);
+        g_free(nevra);
 
         // If parsing failed, move to the next line.
         if (!success)
@@ -264,13 +253,13 @@ sr_abrt_parse_dso_list(const char *text)
             break;
         }
 
-        char *line = sr_strndup(pos, eol - pos);
+        char *line = g_strndup(pos, eol - pos);
         pos = strrchr(line, ' ');
         if (!pos)
         {
             pos = eol;
             sr_rpm_package_free(dso_package, true);
-            free(line);
+            g_free(line);
             continue;
         }
 
@@ -279,7 +268,7 @@ sr_abrt_parse_dso_list(const char *text)
         // Parse the package install time.
         int len = sr_parse_uint64((const char**)&pos,
                                   &dso_package->install_time);
-        free(line);
+        g_free(line);
 
         if (len <= 0)
         {
@@ -317,10 +306,10 @@ rpm_pkg_from_dir(struct sr_rpm_package *packages,
     unsigned long epoch = strtoul(epoch_str, NULL, 10);
     if (epoch == ULONG_MAX)
     {
-        *error_message = sr_asprintf("Epoch '%s' is not a number", epoch_str);
+        *error_message = g_strdup_printf("Epoch '%s' is not a number", epoch_str);
         return NULL;
     }
-    free(epoch_str);
+    g_free(epoch_str);
 
 
     packages->epoch = (uint32_t)epoch;
@@ -366,7 +355,7 @@ rpm_dso_list_from_dir(struct sr_rpm_package *packages,
             packages = sr_rpm_package_uniq(packages);
         }
 
-        free(dso_list_contents);
+        g_free(dso_list_contents);
     }
 
     return packages;
@@ -389,7 +378,7 @@ rpm_interpreter_from_dir(struct sr_rpm_package *packages,
                                                   &interpreter_package->release,
                                                   &interpreter_package->architecture);
 
-        free(interpreter_str);
+        g_free(interpreter_str);
         if (success)
             packages = sr_rpm_package_append(packages, interpreter_package);
         else
@@ -408,7 +397,7 @@ file_exist(const char *directory, const char *filename)
     if (access(path, F_OK) == 0)
         retval = true;
 
-    free(path);
+    g_free(path);
     return retval;
 }
 
@@ -457,7 +446,7 @@ desktop_from_dir(const char *directory,
     char *desktop = strstr(environ_contents, "DESKTOP_SESSION=");
     if (!desktop)
     {
-        free(environ_contents);
+        g_free(environ_contents);
         return NULL;
     }
 
@@ -465,7 +454,7 @@ desktop_from_dir(const char *directory,
        the very first variable or preceeded by a newline */
     if (desktop != environ_contents && *(desktop - 1) != '\n')
     {
-        free(environ_contents);
+        g_free(environ_contents);
         return NULL;
     }
 
@@ -473,8 +462,8 @@ desktop_from_dir(const char *directory,
     char *newline = strchrnul(desktop, '\n');
     *newline = '\0';
 
-    char *result = sr_strdup(desktop);
-    free(environ_contents);
+    char *result = g_strdup(desktop);
+    g_free(environ_contents);
 
     return result;
 }
@@ -496,25 +485,25 @@ occurrences_from_dir(const char *directory,
     if ((errno == ERANGE && (count == LONG_MAX || count == LONG_MIN))
             || (errno != 0 && count == 0))
     {
-        *error_message = sr_asprintf("'count' file does not contain number: %s", strerror(errno));
+        *error_message = g_strdup_printf("'count' file does not contain number: %s", strerror(errno));
         goto finito;
     }
 
     if (endptr == count_contents)
     {
-        *error_message = sr_asprintf("'count' file contains no digits: '%s'", count_contents);
+        *error_message = g_strdup_printf("'count' file contains no digits: '%s'", count_contents);
         goto finito;
     }
 
     if (*endptr != '\0')
     {
-        *error_message = sr_asprintf("'count' file has non-digit suffix: '%s'", endptr);
+        *error_message = g_strdup_printf("'count' file has non-digit suffix: '%s'", endptr);
         goto finito;
     }
 
     if (count < 0 || count > UINT32_MAX)
     {
-        *error_message = sr_asprintf("'count' file is out of range: '%s'", count_contents);
+        *error_message = g_strdup_printf("'count' file is out of range: '%s'", count_contents);
         goto finito;
     }
 
@@ -522,7 +511,7 @@ occurrences_from_dir(const char *directory,
     result = true;
 
 finito:
-    free(count_contents);
+    g_free(count_contents);
     return result;
 }
 
@@ -545,9 +534,9 @@ get_component(const char *directory,
             else
                 basename = executable;
 
-            char *artificial_component = sr_asprintf("%s-unpackaged", basename);
+            char *artificial_component = g_strdup_printf("%s-unpackaged", basename);
 
-            free(executable);
+            g_free(executable);
             return artificial_component;
         }
     }
@@ -566,7 +555,7 @@ sr_abrt_operating_system_from_dir(const char *directory,
     if (osinfo_contents)
     {
         success = sr_operating_system_parse_etc_os_release(osinfo_contents, os);
-        free(osinfo_contents);
+        g_free(osinfo_contents);
     }
 
     /* fall back to os_release if parsing os_info fails */
@@ -579,14 +568,14 @@ sr_abrt_operating_system_from_dir(const char *directory,
             success = sr_operating_system_parse_etc_system_release(release_contents,
                                                                    &os->name,
                                                                    &os->version);
-            free(release_contents);
+            g_free(release_contents);
         }
     }
 
     if (!success)
     {
         sr_operating_system_free(os);
-        *error_message = sr_strdup("Failed to parse operating system release string");
+        *error_message = g_strdup("Failed to parse operating system release string");
         return NULL;
     }
 
@@ -618,7 +607,7 @@ sr_abrt_report_from_dir(const char *directory,
     }
 
     report->report_type = sr_abrt_type_from_type(type_contents);
-    free(type_contents);
+    g_free(type_contents);
 
     /* Operating system. */
     report->operating_system = sr_abrt_operating_system_from_dir(
@@ -661,7 +650,7 @@ sr_abrt_report_from_dir(const char *directory,
         report->stacktrace = (struct sr_stacktrace *)sr_core_stacktrace_from_json_text(
                 core_backtrace_contents, error_message);
 
-        free(core_backtrace_contents);
+        g_free(core_backtrace_contents);
         if (!report->stacktrace)
         {
             sr_report_free(report);
@@ -688,7 +677,7 @@ sr_abrt_report_from_dir(const char *directory,
             &contents_pointer,
             &location);
 
-        free(backtrace_contents);
+        g_free(backtrace_contents);
         if (!report->stacktrace)
         {
             *error_message = sr_location_to_string(&location);
@@ -728,7 +717,7 @@ sr_abrt_report_from_dir(const char *directory,
         stacktrace->version = kernel_contents;
         report->stacktrace = (struct sr_stacktrace *)stacktrace;
 
-        free(backtrace_contents);
+        g_free(backtrace_contents);
         if (!report->stacktrace)
         {
             *error_message = sr_location_to_string(&location);
@@ -755,7 +744,7 @@ sr_abrt_report_from_dir(const char *directory,
             &contents_pointer,
             &location);
 
-        free(backtrace_contents);
+        g_free(backtrace_contents);
         if (!report->stacktrace)
         {
             *error_message = sr_location_to_string(&location);
@@ -783,7 +772,7 @@ sr_abrt_report_from_dir(const char *directory,
             &contents_pointer,
             &location);
 
-        free(backtrace_contents);
+        g_free(backtrace_contents);
         if (!report->stacktrace)
         {
             *error_message = sr_location_to_string(&location);
@@ -816,7 +805,7 @@ sr_abrt_report_from_dir(const char *directory,
         sr_js_platform_t platform = sr_js_platform_from_string(analyzer_contents,
                                                                NULL,
                                                                error_message);
-        free(analyzer_contents);
+        g_free(analyzer_contents);
 
         struct sr_location location;
         sr_location_init(&location);
@@ -835,7 +824,7 @@ sr_abrt_report_from_dir(const char *directory,
             /* This message should help maintainers when a new platform
              * appears. */
             warn("Have to try to guess JavaScript platform: %s", *error_message);
-            free(*error_message);
+            g_free(*error_message);
             *error_message = NULL;
 
             report->stacktrace = (struct sr_stacktrace *)sr_js_stacktrace_parse(
@@ -843,7 +832,7 @@ sr_abrt_report_from_dir(const char *directory,
                 &location);
         }
 
-        free(backtrace_contents);
+        g_free(backtrace_contents);
         if (!report->stacktrace)
         {
             *error_message = sr_location_to_string(&location);
