@@ -20,9 +20,8 @@
 #include "elves.h"
 #include "utils.h"
 #include "config.h"
-#include "strbuf.h"
 
-#if (defined HAVE_DWARF_H && defined HAVE_ELFUTILS_LIBDW_H && defined HAVE_LIBELF_H && defined HAVE_GELF_H && defined HAVE_LIBELF)
+#if (defined HAVE_LIBDW && defined HAVE_LIBELF)
 #  define WITH_ELFUTILS
 #endif
 
@@ -36,6 +35,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <inttypes.h>
+#include <glib.h>
 
 /**
  * Finds a section by its name in an ELF file.
@@ -49,7 +49,7 @@
  *   Save the section header here. Cannot be NULL.
  * @param error_message
  *   Will be filled by an error message if the function fails (returns
- *   zero).  Caller is responsible for calling free() on the string
+ *   zero).  Caller is responsible for calling g_free() on the string
  *   pointer.  If function succeeds, the pointer is not touched by the
  *   function.
  * @returns
@@ -66,7 +66,7 @@ find_elf_section_by_name(Elf *elf,
     size_t shdr_string_index;
     if (0 != elf_getshdrstrndx(elf, &shdr_string_index))
     {
-        *error_message = sr_asprintf("elf_getshdrstrndx failed");
+        *error_message = g_strdup_printf("elf_getshdrstrndx failed");
         return 0;
     }
 
@@ -80,7 +80,7 @@ find_elf_section_by_name(Elf *elf,
         GElf_Shdr shdr;
         if (gelf_getshdr(section, &shdr) != &shdr)
         {
-            *error_message = sr_asprintf("gelf_getshdr failed");
+            *error_message = g_strdup_printf("gelf_getshdr failed");
             return 0;
         }
 
@@ -90,7 +90,7 @@ find_elf_section_by_name(Elf *elf,
 
         if (!current_section_name)
         {
-            *error_message = sr_asprintf("elf_strptr failed");
+            *error_message = g_strdup_printf("elf_strptr failed");
             return 0;
         }
 
@@ -100,7 +100,7 @@ find_elf_section_by_name(Elf *elf,
             *data_dest = elf_getdata(section, NULL);
             if (!*data_dest)
             {
-                *error_message = sr_asprintf("elf_getdata failed");
+                *error_message = g_strdup_printf("elf_getdata failed");
                 return 0;
             }
 
@@ -110,7 +110,7 @@ find_elf_section_by_name(Elf *elf,
         }
     }
 
-    *error_message = sr_asprintf("Section %s not found", section_name);
+    *error_message = g_strdup_printf("Section %s not found", section_name);
     return 0;
 }
 #endif /* WITH_ELFUTILS */
@@ -124,7 +124,7 @@ sr_elf_get_procedure_linkage_table(const char *filename,
     int fd = open(filename, O_RDONLY);
     if (fd < 0)
     {
-        *error_message = sr_asprintf("Failed to open file %s: %s",
+        *error_message = g_strdup_printf("Failed to open file %s: %s",
                                      filename,
                                      strerror(errno));
 
@@ -135,7 +135,7 @@ sr_elf_get_procedure_linkage_table(const char *filename,
     Elf *elf = elf_begin(fd, ELF_C_READ, NULL);
     if (!elf)
     {
-        *error_message = sr_asprintf("Failed to run elf_begin on file %s: %s",
+        *error_message = g_strdup_printf("Failed to run elf_begin on file %s: %s",
                                      filename,
                                      elf_errmsg(-1));
 
@@ -154,11 +154,11 @@ sr_elf_get_procedure_linkage_table(const char *filename,
                                                         &find_section_error_message);
     if (0 == plt_section_index)
     {
-        *error_message = sr_asprintf("Failed to find .plt section for %s: %s",
+        *error_message = g_strdup_printf("Failed to find .plt section for %s: %s",
                                      filename,
                                      find_section_error_message);
 
-        free(find_section_error_message);
+        g_free(find_section_error_message);
         elf_end(elf);
         close(fd);
         return NULL;
@@ -176,7 +176,7 @@ sr_elf_get_procedure_linkage_table(const char *filename,
     {
         if (gelf_getshdr(section, &shdr) != &shdr)
         {
-            *error_message = sr_asprintf("gelf_getshdr failed for %s: %s",
+            *error_message = g_strdup_printf("gelf_getshdr failed for %s: %s",
                                          filename,
                                          elf_errmsg(-1));
 
@@ -191,7 +191,7 @@ sr_elf_get_procedure_linkage_table(const char *filename,
             rela_plt_data = elf_getdata(section, NULL);
             if (!rela_plt_data)
             {
-                *error_message = sr_asprintf("elf_getdata failed for %s: %s",
+                *error_message = g_strdup_printf("elf_getdata failed for %s: %s",
                                              filename,
                                              elf_errmsg(-1));
 
@@ -204,7 +204,7 @@ sr_elf_get_procedure_linkage_table(const char *filename,
             Elf_Scn *symbol_section = elf_getscn(elf, shdr.sh_link);
             if (!symbol_section)
             {
-                *error_message = sr_asprintf("elf_getscn failed for %s: %s",
+                *error_message = g_strdup_printf("elf_getscn failed for %s: %s",
                                              filename,
                                              elf_errmsg(-1));
 
@@ -216,7 +216,7 @@ sr_elf_get_procedure_linkage_table(const char *filename,
             plt_symbols = elf_getdata(symbol_section, NULL);
             if (!plt_symbols)
             {
-                *error_message = sr_asprintf("elf_getdata failed for %s: %s",
+                *error_message = g_strdup_printf("elf_getdata failed for %s: %s",
                                              filename,
                                              elf_errmsg(-1));
 
@@ -228,7 +228,7 @@ sr_elf_get_procedure_linkage_table(const char *filename,
             /* Get string table for the symbol table. */
             if (gelf_getshdr(symbol_section, &shdr) != &shdr)
             {
-                *error_message = sr_asprintf("gelf_getshdr failed for %s: %s",
+                *error_message = g_strdup_printf("gelf_getshdr failed for %s: %s",
                                              filename,
                                              elf_errmsg(-1));
 
@@ -244,7 +244,7 @@ sr_elf_get_procedure_linkage_table(const char *filename,
 
     if (0 == stringtable)
     {
-        *error_message = sr_asprintf("Unable to read symbol table for .plt for file %s",
+        *error_message = g_strdup_printf("Unable to read symbol table for .plt for file %s",
                                      filename);
 
         elf_end(elf);
@@ -279,7 +279,7 @@ sr_elf_get_procedure_linkage_table(const char *filename,
         GElf_Rela rela;
         if (gelf_getrela(rela_plt_data, *plt_index, &rela) != &rela)
         {
-            *error_message = sr_asprintf("gelf_getrela failed for %s: %s",
+            *error_message = g_strdup_printf("gelf_getrela failed for %s: %s",
                                          filename,
                                          elf_errmsg(-1));
 
@@ -292,7 +292,7 @@ sr_elf_get_procedure_linkage_table(const char *filename,
         GElf_Sym symb;
         if (gelf_getsym(plt_symbols, GELF_R_SYM(rela.r_info), &symb) != &symb)
         {
-            *error_message = sr_asprintf("gelf_getsym failed for %s: %s",
+            *error_message = g_strdup_printf("gelf_getsym failed for %s: %s",
                                          filename,
                                          elf_errmsg(-1));
 
@@ -302,8 +302,8 @@ sr_elf_get_procedure_linkage_table(const char *filename,
             return NULL;
         }
 
-        struct sr_elf_plt_entry *entry = sr_malloc(sizeof(struct sr_elf_plt_entry));
-        entry->symbol_name = sr_strdup(elf_strptr(elf, stringtable,
+        struct sr_elf_plt_entry *entry = g_malloc(sizeof(*entry));
+        entry->symbol_name = g_strdup(elf_strptr(elf, stringtable,
                                                   symb.st_name));
 
         entry->address = (uint64_t)(plt_base + plt_offset);
@@ -322,7 +322,7 @@ sr_elf_get_procedure_linkage_table(const char *filename,
     close(fd);
     return result;
 #else /* WITH_ELFUTILS */
-    *error_message = sr_asprintf("satyr compiled without elfutils");
+    *error_message = g_strdup_printf("satyr compiled without elfutils");
     return NULL;
 #endif /* WITH_ELFUTILS */
 }
@@ -334,8 +334,8 @@ sr_elf_procedure_linkage_table_free(struct sr_elf_plt_entry *entries)
     {
         struct sr_elf_plt_entry *entry = entries;
         entries = entry->next;
-        free(entry->symbol_name);
-        free(entry);
+        g_free(entry->symbol_name);
+        g_free(entry);
     }
 }
 
@@ -395,7 +395,7 @@ cie_free(struct cie *entries)
     {
         struct cie *entry = entries;
         entries = entry->next;
-        free(entry);
+        g_free(entry);
     }
 }
 
@@ -409,7 +409,7 @@ read_cie(Dwarf_CFI_Entry *cfi,
     /* Default FDE encoding (i.e. no R in augmentation string) is
      * DW_EH_PE_absptr.
      */
-    struct cie *cie = sr_mallocz(sizeof(struct cie));
+    struct cie *cie = g_malloc0(sizeof(*cie));
     cie->cie_offset = cfi_offset;
     cie->ptr_len = encoded_size(DW_EH_PE_absptr, e_ident);
 
@@ -431,9 +431,9 @@ read_cie(Dwarf_CFI_Entry *cfi,
             cie->ptr_len = encoded_size(*augmentation_data, e_ident);
             if (cie->ptr_len != 4 && cie->ptr_len != 8)
             {
-                *error_message = sr_asprintf("Unknown FDE encoding (CIE %jx)",
+                *error_message = g_strdup_printf("Unknown FDE encoding (CIE %jx)",
                                              (uintmax_t)cfi_offset);
-                free(cie);
+                g_free(cie);
                 return NULL;
             }
 
@@ -449,10 +449,10 @@ read_cie(Dwarf_CFI_Entry *cfi,
             unsigned size = encoded_size(*augmentation_data, e_ident);
             if (0 == size)
             {
-                *error_message = sr_asprintf("Unknown size for personality encoding (CIE %jx)",
+                *error_message = g_strdup_printf("Unknown size for personality encoding (CIE %jx)",
                                              (uintmax_t)cfi_offset);
 
-                free(cie);
+                g_free(cie);
                 return NULL;
             }
 
@@ -460,9 +460,9 @@ read_cie(Dwarf_CFI_Entry *cfi,
             break;
         }
         default:
-            *error_message = sr_asprintf("Unknown augmentation char (CIE %jx)",
+            *error_message = g_strdup_printf("Unknown augmentation char (CIE %jx)",
                                          (uintmax_t)cfi_offset);
-            free(cie);
+            g_free(cie);
             return NULL;
         }
 
@@ -481,15 +481,15 @@ read_cie(Dwarf_CFI_Entry *cfi,
 static uint64_t
 fde_read_address(const uint8_t *p, unsigned len)
 {
-    int i;
     union {
         uint8_t b[8];
         /* uint16_t n2; */
         uint32_t n4;
         uint64_t n8;
     } u;
+    u.n8 = 0;
 
-    for (i = 0; i < len; i++)
+    for (unsigned i = 0; i < len; i++)
         u.b[i] = *p++;
 
     return (len == 4 ? (uint64_t)u.n4 : u.n8);
@@ -505,7 +505,7 @@ sr_elf_get_eh_frame(const char *filename,
     int fd = open(filename, O_RDONLY);
     if (fd < 0)
     {
-        *error_message = sr_asprintf("Failed to open file %s: %s",
+        *error_message = g_strdup_printf("Failed to open file %s: %s",
                                      filename,
                                      strerror(errno));
         return NULL;
@@ -515,7 +515,7 @@ sr_elf_get_eh_frame(const char *filename,
     Elf *elf = elf_begin(fd, ELF_C_READ, NULL);
     if (!elf)
     {
-        *error_message = sr_asprintf("Failed to run elf_begin on file %s: %s",
+        *error_message = g_strdup_printf("Failed to run elf_begin on file %s: %s",
                                      filename,
                                      elf_errmsg(-1));
         close(fd);
@@ -525,7 +525,7 @@ sr_elf_get_eh_frame(const char *filename,
     unsigned char *e_ident = (unsigned char *)elf_getident(elf, NULL);
     if (!e_ident)
     {
-        *error_message = sr_asprintf("elf_getident failed for %s: %s",
+        *error_message = g_strdup_printf("elf_getident failed for %s: %s",
                                      filename,
                                      elf_errmsg(-1));
         elf_end(elf);
@@ -543,11 +543,11 @@ sr_elf_get_eh_frame(const char *filename,
                                   &shdr,
                                   &find_section_error_message))
     {
-        *error_message = sr_asprintf("Failed to find .eh_frame section for %s: %s",
+        *error_message = g_strdup_printf("Failed to find .eh_frame section for %s: %s",
                                      filename,
                                      find_section_error_message);
 
-        free(find_section_error_message);
+        g_free(find_section_error_message);
         elf_end(elf);
         close(fd);
         return NULL;
@@ -562,7 +562,7 @@ sr_elf_get_eh_frame(const char *filename,
     size_t phnum;
     if (elf_getphdrnum(elf, &phnum) != 0)
     {
-        *error_message = sr_asprintf("elf_getphdrnum failed for %s: %s",
+        *error_message = g_strdup_printf("elf_getphdrnum failed for %s: %s",
                                      filename,
                                      elf_errmsg(-1));
         elf_end(elf);
@@ -570,14 +570,13 @@ sr_elf_get_eh_frame(const char *filename,
         return NULL;
     }
 
-    uint64_t exec_base = -1;
-    int i;
-    for (i = 0; i < phnum; i++)
+    uint64_t exec_base = UINT64_MAX;
+    for (unsigned i = 0; i < phnum; i++)
     {
         GElf_Phdr phdr;
         if (gelf_getphdr(elf, i, &phdr) != &phdr)
         {
-            *error_message = sr_asprintf("gelf_getphdr failed for %s: %s",
+            *error_message = g_strdup_printf("gelf_getphdr failed for %s: %s",
                                          filename,
                                          elf_errmsg(-1));
             elf_end(elf);
@@ -592,9 +591,9 @@ sr_elf_get_eh_frame(const char *filename,
         }
     }
 
-    if (-1 == exec_base)
+    if (exec_base == UINT64_MAX)
     {
-        *error_message = sr_asprintf("Can't determine executable base for %s",
+        *error_message = g_strdup_printf("Can't determine executable base for %s",
                                      filename);
 
         elf_end(elf);
@@ -643,7 +642,7 @@ sr_elf_get_eh_frame(const char *filename,
             if (cfi_offset_next > cfi_offset)
                 continue;
 
-            *error_message = sr_asprintf("dwarf_next_cfi failed for %s: %s",
+            *error_message = g_strdup_printf("dwarf_next_cfi failed for %s: %s",
                                          filename,
                                          dwarf_errmsg(-1));
 
@@ -670,11 +669,11 @@ sr_elf_get_eh_frame(const char *filename,
                                        &cie_error_message);
             if (!cie)
             {
-                *error_message = sr_asprintf("CIE reading failed for %s: %s",
+                *error_message = g_strdup_printf("CIE reading failed for %s: %s",
                                              filename,
                                              cie_error_message);
 
-                free(cie_error_message);
+                g_free(cie_error_message);
                 cie_free(cie_list);
                 sr_elf_eh_frame_free(result);
                 elf_end(elf);
@@ -710,7 +709,7 @@ sr_elf_get_eh_frame(const char *filename,
 
             if (!cie)
             {
-                *error_message = sr_asprintf("CIE not found for FDE %jx in %s",
+                *error_message = g_strdup_printf("CIE not found for FDE %jx in %s",
                                              (uintmax_t)cfi_offset,
                                              filename);
 
@@ -751,7 +750,7 @@ sr_elf_get_eh_frame(const char *filename,
                 initial_location -= exec_base;
             }
 
-            struct sr_elf_fde *fde = sr_malloc(sizeof(struct sr_elf_fde));
+            struct sr_elf_fde *fde = g_malloc(sizeof(*fde));
             fde->exec_base = exec_base;
             fde->start_address = initial_location;
             fde->length = address_range;
@@ -775,7 +774,7 @@ sr_elf_get_eh_frame(const char *filename,
     close(fd);
     return result;
 #else /* WITH_ELFUTILS */
-    *error_message = sr_asprintf("satyr compiled without elfutils");
+    *error_message = g_strdup_printf("satyr compiled without elfutils");
     return NULL;
 #endif /* WITH_ELFUTILS */
 }
@@ -787,7 +786,7 @@ sr_elf_eh_frame_free(struct sr_elf_fde *entries)
     {
         struct sr_elf_fde *entry = entries;
         entries = entry->next;
-        free(entry);
+        g_free(entry);
     }
 }
 
@@ -849,7 +848,7 @@ char *
 sr_elf_fde_to_json(struct sr_elf_fde *fde,
                    bool recursive)
 {
-    struct sr_strbuf *strbuf = sr_strbuf_new();
+    GString *strbuf = g_string_new(NULL);
 
     if (recursive)
     {
@@ -857,36 +856,36 @@ sr_elf_fde_to_json(struct sr_elf_fde *fde,
         while (loop)
         {
             if (loop == fde)
-                sr_strbuf_append_str(strbuf, "[ ");
+                g_string_append(strbuf, "[ ");
             else
-                sr_strbuf_append_str(strbuf, ", ");
+                g_string_append(strbuf, ", ");
 
             char *fde_json = sr_elf_fde_to_json(loop, false);
             char *indented_fde_json = sr_indent_except_first_line(fde_json, 2);
-            sr_strbuf_append_str(strbuf, indented_fde_json);
-            free(indented_fde_json);
-            free(fde_json);
+            g_string_append(strbuf, indented_fde_json);
+            g_free(indented_fde_json);
+            g_free(fde_json);
             loop = loop->next;
             if (loop)
-                sr_strbuf_append_str(strbuf, "\n");
+                g_string_append(strbuf, "\n");
         }
 
-        sr_strbuf_append_str(strbuf, " ]");
+        g_string_append(strbuf, " ]");
     }
     else
     {
         /* Start address. */
-        sr_strbuf_append_strf(strbuf,
+        g_string_append_printf(strbuf,
                               "{   \"start_address\": %"PRIu64"\n",
                               fde->start_address);
 
         /* Length. */
-        sr_strbuf_append_strf(strbuf,
+        g_string_append_printf(strbuf,
                               ",   \"length\": %"PRIu64"\n",
                               fde->length);
 
-        sr_strbuf_append_str(strbuf, "}");
+        g_string_append(strbuf, "}");
     }
 
-    return sr_strbuf_free_nobuf(strbuf);
+    return g_string_free(strbuf, FALSE);
 }
