@@ -19,7 +19,6 @@
     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 #include "java/frame.h"
-#include "strbuf.h"
 #include "location.h"
 #include "utils.h"
 #include "json.h"
@@ -38,10 +37,10 @@
 
 static void
 java_append_bthash_text(struct sr_java_frame *frame, enum sr_bthash_flags flags,
-                        struct sr_strbuf *strbuf);
+                        GString *strbuf);
 static void
 java_append_duphash_text(struct sr_java_frame *frame, enum sr_duphash_flags flags,
-                         struct sr_strbuf *strbuf);
+                         GString *strbuf);
 
 DEFINE_NEXT_FUNC(java_next, struct sr_frame, struct sr_java_frame)
 DEFINE_SET_NEXT_FUNC(java_set_next, struct sr_frame, struct sr_java_frame)
@@ -66,7 +65,7 @@ struct sr_java_frame *
 sr_java_frame_new()
 {
     struct sr_java_frame *frame =
-        sr_malloc(sizeof(*frame));
+        g_malloc(sizeof(*frame));
 
     sr_java_frame_init(frame);
     return frame;
@@ -76,7 +75,7 @@ struct sr_java_frame *
 sr_java_frame_new_exception()
 {
     struct sr_java_frame *frame =
-        sr_malloc(sizeof(*frame));
+        g_malloc(sizeof(*frame));
 
     sr_java_frame_init(frame);
     frame->is_exception = true;
@@ -96,11 +95,11 @@ sr_java_frame_free(struct sr_java_frame *frame)
     if (!frame)
         return;
 
-    free(frame->file_name);
-    free(frame->name);
-    free(frame->class_path);
-    free(frame->message);
-    free(frame);
+    g_free(frame->file_name);
+    g_free(frame->name);
+    g_free(frame->class_path);
+    g_free(frame->message);
+    g_free(frame);
 }
 
 void
@@ -140,16 +139,16 @@ sr_java_frame_dup(struct sr_java_frame *frame, bool siblings)
 
     /* Duplicate all strings. */
     if (result->file_name)
-        result->file_name = sr_strdup(result->file_name);
+        result->file_name = g_strdup(result->file_name);
 
     if (result->name)
-        result->name = sr_strdup(result->name);
+        result->name = g_strdup(result->name);
 
     if (result->class_path)
-        result->class_path = sr_strdup(result->class_path);
+        result->class_path = g_strdup(result->class_path);
 
     if (result->message)
-        result->message = sr_strdup(result->message);
+        result->message = g_strdup(result->message);
 
     return result;
 }
@@ -161,7 +160,7 @@ sr_java_frame_cmp(struct sr_java_frame *frame1,
     if (frame1->is_exception != frame2->is_exception)
         return frame1->is_exception ? 1 : -1;
 
-    int res = sr_strcmp0(frame1->name, frame2->name);
+    int res = g_strcmp0(frame1->name, frame2->name);
     if (res != 0)
         return res;
 
@@ -170,11 +169,11 @@ sr_java_frame_cmp(struct sr_java_frame *frame1,
         return 0;
 
     /* Method call comparsion */
-    res = sr_strcmp0(frame1->class_path, frame2->class_path);
+    res = g_strcmp0(frame1->class_path, frame2->class_path);
     if (res != 0)
         return res;
 
-    res = sr_strcmp0(frame1->file_name, frame2->file_name);
+    res = g_strcmp0(frame1->file_name, frame2->file_name);
     if (res != 0)
         return res;
 
@@ -188,7 +187,7 @@ int
 sr_java_frame_cmp_distance(struct sr_java_frame *frame1,
                            struct sr_java_frame *frame2)
 {
-    int res = sr_strcmp0(frame1->name, frame2->name);
+    int res = g_strcmp0(frame1->name, frame2->name);
     if (res != 0)
         return res;
 
@@ -212,46 +211,46 @@ sr_java_frame_append(struct sr_java_frame *dest,
 
 void
 sr_java_frame_append_to_str(struct sr_java_frame *frame,
-                            struct sr_strbuf *dest)
+                            GString *dest)
 {
     if (frame->is_exception)
     {
         if (frame->name)
-            sr_strbuf_append_str(dest, frame->name);
+            g_string_append(dest, frame->name);
 
         if (frame->message)
-            sr_strbuf_append_strf(dest, ": %s", frame->message);
+            g_string_append_printf(dest, ": %s", frame->message);
     }
     else
     {
-        sr_strbuf_append_strf(dest, "\tat %s(",
+        g_string_append_printf(dest, "\tat %s(",
                               frame->name ? frame->name : "");
 
         if (frame->is_native)
-            sr_strbuf_append_str(dest, SR_JF_MARK_NATIVE_METHOD);
+            g_string_append(dest, SR_JF_MARK_NATIVE_METHOD);
         else if (!frame->file_name)
-            sr_strbuf_append_str(dest, SR_JF_MARK_UNKNOWN_SOURCE);
+            g_string_append(dest, SR_JF_MARK_UNKNOWN_SOURCE);
         else
-            sr_strbuf_append_str(dest, frame->file_name);
+            g_string_append(dest, frame->file_name);
 
         /* YES even if the frame is native method or source is unknown */
         /* WHY? Because it was parsed in this form */
         /* Ooops! Maybe the source file was empty string. Don't care! */
         if (frame->file_line)
-            sr_strbuf_append_strf(dest, ":%"PRIu32, frame->file_line);
+            g_string_append_printf(dest, ":%"PRIu32, frame->file_line);
 
-        sr_strbuf_append_str(dest, ")");
+        g_string_append(dest, ")");
 
         if (!frame->class_path)
-            sr_strbuf_append_str(dest, " [unknown]");
+            g_string_append(dest, " [unknown]");
         else
         {
-            sr_strbuf_append_str(dest, " [");
+            g_string_append(dest, " [");
 
             if (strchrnul(frame->class_path,':') > strchrnul(frame->class_path, '/'))
-                sr_strbuf_append_str(dest, "file:");
+                g_string_append(dest, "file:");
 
-            sr_strbuf_append_strf(dest, "%s]", frame->class_path);
+            g_string_append_printf(dest, "%s]", frame->class_path);
         }
     }
 }
@@ -291,7 +290,7 @@ sr_java_frame_parse_exception(const char **input,
     }
 
     struct sr_java_frame *exception = sr_java_frame_new_exception();
-    exception->name = sr_strndup(mark, cursor - mark);
+    exception->name = g_strndup(mark, cursor - mark);
 
     /* : foo */
     if (*cursor == ':')
@@ -308,7 +307,7 @@ sr_java_frame_parse_exception(const char **input,
         sr_location_add(location, 0, sr_skip_char_cspan(&cursor, "\n"));
 
         if (mark != cursor)
-            exception->message = sr_strndup(mark, cursor - mark);
+            exception->message = g_strndup(mark, cursor - mark);
     }
     else
     {
@@ -468,7 +467,10 @@ const char *sr_java_frame_parse_frame_url(struct sr_java_frame *frame, const cha
         sr_location_add(location, 0, sr_skip_char_cspan(&cursor, path_stop));
 
         if (mark != cursor)
-            frame->class_path = sr_strndup(mark, cursor - mark);
+        {
+            frame->class_path = g_strndup(mark, cursor - mark);
+            frame->class_path = anonymize_path(frame->class_path);
+        }
     }
 
     if (*cursor != ']' && *cursor != '\n')
@@ -506,7 +508,7 @@ sr_java_frame_parse(const char **input,
     struct sr_java_frame *frame = sr_java_frame_new();
 
     if (cursor != mark)
-        frame->name = sr_strndup(mark, cursor - mark);
+        frame->name = g_strndup(mark, cursor - mark);
 
     /* (SimpleTest.java:36) [file:/usr/lib/java/foo.class] */
     if (*cursor == '(')
@@ -522,8 +524,11 @@ sr_java_frame_parse(const char **input,
             if (sr_java_frame_parse_is_native_method(mark))
                 frame->is_native = true;
             else if (!sr_java_frame_parse_is_unknown_source(mark))
+            {
                 /* DO NOT set file_name if input says that source isn't known */
-                frame->file_name = sr_strndup(mark, cursor - mark);
+                frame->file_name = g_strndup(mark, cursor - mark);
+                frame->file_name = anonymize_path(frame->file_name);
+            }
         }
 
         if (*cursor == ':')
@@ -560,25 +565,25 @@ sr_java_frame_parse(const char **input,
 char *
 sr_java_frame_to_json(struct sr_java_frame *frame)
 {
-    struct sr_strbuf *strbuf = sr_strbuf_new();
+    GString *strbuf = g_string_new(NULL);
 
     /* Name. */
     if (frame->name)
     {
-        sr_strbuf_append_str(strbuf, ",   \"name\": ");
+        g_string_append(strbuf, ",   \"name\": ");
         sr_json_append_escaped(strbuf, frame->name);
-        sr_strbuf_append_str(strbuf, "\n");
+        g_string_append(strbuf, "\n");
     }
 
     /* File name. */
     if (frame->file_name)
     {
-        sr_strbuf_append_str(strbuf, ",   \"file_name\": ");
+        g_string_append(strbuf, ",   \"file_name\": ");
         sr_json_append_escaped(strbuf, frame->file_name);
-        sr_strbuf_append_str(strbuf, "\n");
+        g_string_append(strbuf, "\n");
 
         /* File line. */
-        sr_strbuf_append_strf(strbuf,
+        g_string_append_printf(strbuf,
                               ",   \"file_line\": %"PRIu32"\n",
                               frame->file_line);
     }
@@ -586,38 +591,38 @@ sr_java_frame_to_json(struct sr_java_frame *frame)
     /* Class path. */
     if (frame->class_path)
     {
-        sr_strbuf_append_str(strbuf, ",   \"class_path\": ");
+        g_string_append(strbuf, ",   \"class_path\": ");
         sr_json_append_escaped(strbuf, frame->class_path);
-        sr_strbuf_append_str(strbuf, "\n");
+        g_string_append(strbuf, "\n");
     }
 
     /* Is native? */
-    sr_strbuf_append_strf(strbuf,
+    g_string_append_printf(strbuf,
                           ",   \"is_native\": %s\n",
                           frame->is_native ? "true" : "false");
 
     /* Is exception? */
-    sr_strbuf_append_strf(strbuf,
+    g_string_append_printf(strbuf,
                           ",   \"is_exception\": %s\n",
                           frame->is_exception ? "true" : "false");
 
     /* Message. */
     if (frame->message)
     {
-        sr_strbuf_append_str(strbuf, ",   \"message\": ");
+        g_string_append(strbuf, ",   \"message\": ");
         sr_json_append_escaped(strbuf, frame->message);
-        sr_strbuf_append_str(strbuf, "\n");
+        g_string_append(strbuf, "\n");
     }
 
-    strbuf->buf[0] = '{';
-    sr_strbuf_append_str(strbuf, "}");
-    return sr_strbuf_free_nobuf(strbuf);
+    strbuf->str[0] = '{';
+    g_string_append(strbuf, "}");
+    return g_string_free(strbuf, FALSE);
 }
 
 struct sr_java_frame *
-sr_java_frame_from_json(struct sr_json_value *root, char **error_message)
+sr_java_frame_from_json(json_object *root, char **error_message)
 {
-    if (!JSON_CHECK_TYPE(root, SR_JSON_OBJECT, "frame"))
+    if (!json_check_type(root, json_type_object, "frame", error_message))
         return NULL;
 
     struct sr_java_frame *result = sr_java_frame_new();
@@ -642,9 +647,9 @@ sr_java_frame_from_json(struct sr_json_value *root, char **error_message)
 
 static void
 java_append_bthash_text(struct sr_java_frame *frame, enum sr_bthash_flags flags,
-                        struct sr_strbuf *strbuf)
+                        GString *strbuf)
 {
-    sr_strbuf_append_strf(strbuf,
+    g_string_append_printf(strbuf,
                           "%s, %s, %"PRIu32", %s, %d, %d, %s\n",
                           OR_UNKNOWN(frame->name),
                           OR_UNKNOWN(frame->file_name),
@@ -657,12 +662,12 @@ java_append_bthash_text(struct sr_java_frame *frame, enum sr_bthash_flags flags,
 
 static void
 java_append_duphash_text(struct sr_java_frame *frame, enum sr_duphash_flags flags,
-                         struct sr_strbuf *strbuf)
+                         GString *strbuf)
 {
     if (frame->name)
-        sr_strbuf_append_strf(strbuf, "%s\n", frame->name);
+        g_string_append_printf(strbuf, "%s\n", frame->name);
     else
-        sr_strbuf_append_strf(strbuf, "%s/%s:%"PRIu32"\n",
+        g_string_append_printf(strbuf, "%s/%s:%"PRIu32"\n",
                               OR_UNKNOWN(frame->class_path),
                               OR_UNKNOWN(frame->file_name),
                               frame->file_line);
