@@ -19,7 +19,6 @@
 */
 #include "koops/frame.h"
 #include "utils.h"
-#include "strbuf.h"
 #include "json.h"
 #include "generic_frame.h"
 #include "thread.h"
@@ -34,10 +33,10 @@
 
 static void
 koops_append_bthash_text(struct sr_koops_frame *frame, enum sr_bthash_flags flags,
-                         struct sr_strbuf *strbuf);
+                         GString *strbuf);
 static void
 koops_append_duphash_text(struct sr_koops_frame *frame, enum sr_duphash_flags flags,
-                          struct sr_strbuf *strbuf);
+                          GString *strbuf);
 
 DEFINE_NEXT_FUNC(koops_next, struct sr_frame, struct sr_koops_frame)
 DEFINE_SET_NEXT_FUNC(koops_set_next, struct sr_frame, struct sr_koops_frame)
@@ -62,7 +61,7 @@ struct sr_koops_frame *
 sr_koops_frame_new()
 {
     struct sr_koops_frame *frame =
-        sr_malloc(sizeof(struct sr_koops_frame));
+        g_malloc(sizeof(*frame));
 
     sr_koops_frame_init(frame);
     return frame;
@@ -81,12 +80,12 @@ sr_koops_frame_free(struct sr_koops_frame *frame)
     if (!frame)
         return;
 
-    free(frame->function_name);
-    free(frame->module_name);
-    free(frame->from_function_name);
-    free(frame->from_module_name);
-    free(frame->special_stack);
-    free(frame);
+    g_free(frame->function_name);
+    g_free(frame->module_name);
+    g_free(frame->from_function_name);
+    g_free(frame->from_module_name);
+    g_free(frame->special_stack);
+    g_free(frame);
 }
 
 struct sr_koops_frame *
@@ -106,19 +105,19 @@ sr_koops_frame_dup(struct sr_koops_frame *frame, bool siblings)
 
     /* Duplicate all strings. */
     if (result->function_name)
-        result->function_name = sr_strdup(result->function_name);
+        result->function_name = g_strdup(result->function_name);
 
     if (result->module_name)
-        result->module_name = sr_strdup(result->module_name);
+        result->module_name = g_strdup(result->module_name);
 
     if (result->from_function_name)
-        result->from_function_name = sr_strdup(result->from_function_name);
+        result->from_function_name = g_strdup(result->from_function_name);
 
     if (result->from_module_name)
-        result->from_module_name = sr_strdup(result->from_module_name);
+        result->from_module_name = g_strdup(result->from_module_name);
 
     if (result->special_stack)
-        result->special_stack = sr_strdup(result->special_stack);
+        result->special_stack = g_strdup(result->special_stack);
 
     return result;
 }
@@ -138,7 +137,7 @@ sr_koops_frame_cmp(struct sr_koops_frame *frame1,
         return reliable;
 
     /* Function name. */
-    int function_name = sr_strcmp0(frame1->function_name,
+    int function_name = g_strcmp0(frame1->function_name,
                                    frame2->function_name);
 
     if (function_name != 0)
@@ -155,7 +154,7 @@ sr_koops_frame_cmp(struct sr_koops_frame *frame1,
         return function_length;
 
     /* Module name. */
-    int module_name = sr_strcmp0(frame1->module_name,
+    int module_name = g_strcmp0(frame1->module_name,
                                  frame2->module_name);
 
     if (module_name != 0)
@@ -167,7 +166,7 @@ sr_koops_frame_cmp(struct sr_koops_frame *frame1,
         return from_address;
 
     /* From function name. */
-    int from_function_name = sr_strcmp0(frame1->from_function_name,
+    int from_function_name = g_strcmp0(frame1->from_function_name,
                                         frame2->from_function_name);
 
     if (from_function_name != 0)
@@ -184,14 +183,14 @@ sr_koops_frame_cmp(struct sr_koops_frame *frame1,
         return from_function_length;
 
     /* From module name. */
-    int from_module_name = sr_strcmp0(frame1->from_module_name,
+    int from_module_name = g_strcmp0(frame1->from_module_name,
                                       frame2->from_module_name);
 
     if (from_module_name != 0)
         return from_module_name;
 
     /* Special stack. */
-    int special_stack = sr_strcmp0(frame1->special_stack,
+    int special_stack = g_strcmp0(frame1->special_stack,
                                    frame2->special_stack);
 
     if (special_stack != 0)
@@ -205,7 +204,7 @@ sr_koops_frame_cmp_distance(struct sr_koops_frame *frame1,
                             struct sr_koops_frame *frame2)
 {
     /* Function. */
-    int function_name = sr_strcmp0(frame1->function_name,
+    int function_name = g_strcmp0(frame1->function_name,
                                     frame2->function_name);
     if (function_name != 0)
         return function_name;
@@ -509,7 +508,7 @@ sr_koops_parse_module_name(const char **input,
 
     if (!sr_skip_char(&local_input, ']'))
     {
-        free(*module_name);
+        g_free(*module_name);
         *module_name = NULL;
         return false;
     }
@@ -554,7 +553,7 @@ sr_koops_parse_function(const char **input,
 
         if (!sr_skip_char(&local_input, '/'))
         {
-            free(*function_name);
+            g_free(*function_name);
             *function_name = NULL;
             return false;
         }
@@ -575,11 +574,11 @@ sr_koops_parse_function(const char **input,
 
     if (parenthesis && !sr_skip_char(&local_input, ')'))
     {
-        free(*function_name);
+        g_free(*function_name);
         *function_name = NULL;
         if (has_module)
         {
-            free(*module_name);
+            g_free(*module_name);
             *module_name = NULL;
         }
 
@@ -593,86 +592,86 @@ sr_koops_parse_function(const char **input,
 char *
 sr_koops_frame_to_json(struct sr_koops_frame *frame)
 {
-    struct sr_strbuf *strbuf = sr_strbuf_new();
+    GString *strbuf = g_string_new(NULL);
 
     if (frame->address != 0)
     {
-        sr_strbuf_append_strf(strbuf,
+        g_string_append_printf(strbuf,
                               "{   \"address\": %"PRIu64"\n",
                               frame->address);
     }
 
-    sr_strbuf_append_strf(strbuf,
+    g_string_append_printf(strbuf,
                           "%s   \"reliable\": %s\n",
                           frame->address == 0 ? "{" : ",",
                           frame->reliable ? "true" : "false");
 
     if (frame->function_name)
     {
-        sr_strbuf_append_str(strbuf, ",   \"function_name\": ");
+        g_string_append(strbuf, ",   \"function_name\": ");
         sr_json_append_escaped(strbuf, frame->function_name);
-        sr_strbuf_append_str(strbuf, "\n");
+        g_string_append(strbuf, "\n");
     }
 
-    sr_strbuf_append_strf(strbuf,
+    g_string_append_printf(strbuf,
                           ",   \"function_offset\": %"PRIu64"\n",
                           frame->function_offset);
 
-    sr_strbuf_append_strf(strbuf,
+    g_string_append_printf(strbuf,
                           ",   \"function_length\": %"PRIu64"\n",
                           frame->function_length);
 
     if (frame->module_name)
     {
-        sr_strbuf_append_str(strbuf, ",   \"module_name\": ");
+        g_string_append(strbuf, ",   \"module_name\": ");
         sr_json_append_escaped(strbuf, frame->module_name);
-        sr_strbuf_append_str(strbuf, "\n");
+        g_string_append(strbuf, "\n");
     }
 
     if (frame->from_address != 0)
     {
-        sr_strbuf_append_strf(strbuf,
+        g_string_append_printf(strbuf,
                               ",   \"from_address\": %"PRIu64"\n",
                               frame->from_address);
     }
 
     if (frame->from_function_name)
     {
-        sr_strbuf_append_str(strbuf, ",   \"from_function_name\": ");
+        g_string_append(strbuf, ",   \"from_function_name\": ");
         sr_json_append_escaped(strbuf, frame->from_function_name);
-        sr_strbuf_append_str(strbuf, "\n");
+        g_string_append(strbuf, "\n");
     }
 
-    sr_strbuf_append_strf(strbuf,
+    g_string_append_printf(strbuf,
                           ",   \"from_function_offset\": %"PRIu64"\n",
                           frame->from_function_offset);
 
-    sr_strbuf_append_strf(strbuf,
+    g_string_append_printf(strbuf,
                           ",   \"from_function_length\": %"PRIu64"\n",
                           frame->from_function_length);
 
     if (frame->from_module_name)
     {
-        sr_strbuf_append_str(strbuf, ",   \"from_module_name\": ");
+        g_string_append(strbuf, ",   \"from_module_name\": ");
         sr_json_append_escaped(strbuf, frame->from_module_name);
-        sr_strbuf_append_str(strbuf, "\n");
+        g_string_append(strbuf, "\n");
     }
 
     if (frame->special_stack)
     {
-        sr_strbuf_append_str(strbuf, ",   \"special_stack\": ");
+        g_string_append(strbuf, ",   \"special_stack\": ");
         sr_json_append_escaped(strbuf, frame->special_stack);
-        sr_strbuf_append_str(strbuf, "\n");
+        g_string_append(strbuf, "\n");
     }
 
-    sr_strbuf_append_str(strbuf, "}");
-    return sr_strbuf_free_nobuf(strbuf);
+    g_string_append(strbuf, "}");
+    return g_string_free(strbuf, FALSE);
 }
 
 struct sr_koops_frame *
-sr_koops_frame_from_json(struct sr_json_value *root, char **error_message)
+sr_koops_frame_from_json(json_object *root, char **error_message)
 {
-    if (!JSON_CHECK_TYPE(root, SR_JSON_OBJECT, "frame"))
+    if (!json_check_type(root, json_type_object, "frame", error_message))
         return NULL;
 
     struct sr_koops_frame *result = sr_koops_frame_new();
@@ -702,24 +701,24 @@ sr_koops_frame_from_json(struct sr_json_value *root, char **error_message)
 
 void
 sr_koops_frame_append_to_str(struct sr_koops_frame *frame,
-                             struct sr_strbuf *str)
+                             GString *str)
 {
     if (frame->special_stack)
-        sr_strbuf_append_strf(str, "[%s] ", frame->special_stack);
+        g_string_append_printf(str, "[%s] ", frame->special_stack);
 
-    sr_strbuf_append_strf(str, "%s%s",
+    g_string_append_printf(str, "%s%s",
                           (frame->reliable ? "" : "? "),
                           (frame->function_name ? frame->function_name : "??"));
 
     if (frame->module_name)
-        sr_strbuf_append_strf(str, " in %s", frame->module_name);
+        g_string_append_printf(str, " in %s", frame->module_name);
 }
 
 static void
 koops_append_bthash_text(struct sr_koops_frame *frame, enum sr_bthash_flags flags,
-                         struct sr_strbuf *strbuf)
+                         GString *strbuf)
 {
-    sr_strbuf_append_strf(strbuf,
+    g_string_append_printf(strbuf,
                           "0x%"PRIx64", %d, %s, 0x%"PRIx64", 0x%"PRIx64", %s, "
                           "0x%"PRIx64", %s, 0x%"PRIx64", 0x%"PRIx64", %s\n",
                           frame->address,
@@ -737,14 +736,14 @@ koops_append_bthash_text(struct sr_koops_frame *frame, enum sr_bthash_flags flag
 
 static void
 koops_append_duphash_text(struct sr_koops_frame *frame, enum sr_duphash_flags flags,
-                         struct sr_strbuf *strbuf)
+                         GString *strbuf)
 {
     /* ABRT's koops hashing skipped unreliable frames entirely */
     if ((flags & SR_DUPHASH_KOOPS_COMPAT) && !frame->reliable)
         return;
 
     if (frame->function_name)
-        sr_strbuf_append_strf(strbuf, "%s\n", frame->function_name);
+        g_string_append_printf(strbuf, "%s\n", frame->function_name);
     else
-        sr_strbuf_append_strf(strbuf, "0x%"PRIx64"\n", frame->address);
+        g_string_append_printf(strbuf, "0x%"PRIx64"\n", frame->address);
 }
