@@ -21,7 +21,6 @@
 #include "koops/frame.h"
 #include "utils.h"
 #include "json.h"
-#include "strbuf.h"
 #include "normalize.h"
 #include "generic_thread.h"
 #include "generic_stacktrace.h"
@@ -61,7 +60,7 @@ struct sr_taint_flag sr_flags[] = {
 
 static void
 koops_append_bthash_text(struct sr_koops_stacktrace *stacktrace,
-                         enum sr_bthash_flags flags, struct sr_strbuf *strbuf);
+                         enum sr_bthash_flags flags, GString *strbuf);
 
 DEFINE_FRAMES_FUNC(koops_frames, struct sr_koops_stacktrace)
 DEFINE_SET_FRAMES_FUNC(koops_set_frames, struct sr_koops_stacktrace)
@@ -106,7 +105,7 @@ struct sr_koops_stacktrace *
 sr_koops_stacktrace_new()
 {
     struct sr_koops_stacktrace *stacktrace =
-        sr_malloc(sizeof(struct sr_koops_stacktrace));
+        g_malloc(sizeof(*stacktrace));
 
     sr_koops_stacktrace_init(stacktrace);
     return stacktrace;
@@ -132,10 +131,12 @@ sr_koops_stacktrace_free(struct sr_koops_stacktrace *stacktrace)
         sr_koops_frame_free(frame);
     }
 
-    free(stacktrace->version);
-    free(stacktrace->raw_oops);
-    free(stacktrace->reason);
-    free(stacktrace);
+    g_strfreev(stacktrace->modules);
+
+    g_free(stacktrace->version);
+    g_free(stacktrace->raw_oops);
+    g_free(stacktrace->reason);
+    g_free(stacktrace);
 }
 
 struct sr_koops_stacktrace *
@@ -148,13 +149,15 @@ sr_koops_stacktrace_dup(struct sr_koops_stacktrace *stacktrace)
         result->frames = sr_koops_frame_dup(result->frames, true);
 
     if (result->version)
-        result->version = sr_strdup(result->version);
+        result->version = g_strdup(result->version);
 
     if (result->raw_oops)
-        result->raw_oops = sr_strdup(result->raw_oops);
+        result->raw_oops = g_strdup(result->raw_oops);
 
     if (result->reason)
-        result->reason = sr_strdup(result->reason);
+        result->reason = g_strdup(result->reason);
+
+    result->modules = g_strdupv(result->modules);
 
     return result;
 }
@@ -317,7 +320,7 @@ parse_alt_stack_start(const char **input)
         !sr_parse_char_cspan(&local_input, "> \t\n", &stack_label) ||
         !sr_skip_char(&local_input, '>'))
     {
-        free(stack_label);
+        g_free(stack_label);
         return NULL;
     }
 
@@ -337,13 +340,13 @@ sr_koops_stacktrace_parse(const char **input,
     char *alt_stack = NULL;
 
     /* Include the raw kerneloops text */
-    stacktrace->raw_oops = sr_strdup(*input);
+    stacktrace->raw_oops = g_strdup(*input);
 
     /* Looks for the "Tainted: " line in the whole input */
     parse_taint_flags(local_input, stacktrace);
 
     /* The "reason" is expected to be the first line of the input */
-    stacktrace->reason = sr_strndup(*input, strcspn(*input, "\n"));
+    stacktrace->reason = g_strndup(*input, strcspn(*input, "\n"));
 
     while (*local_input)
     {
@@ -380,7 +383,7 @@ sr_koops_stacktrace_parse(const char **input,
         /* <IRQ>, <NMI>, ... */
         if (parse_alt_stack_end(&local_input))
         {
-            free(alt_stack);
+            g_free(alt_stack);
             alt_stack = NULL;
         }
 
@@ -394,7 +397,7 @@ sr_koops_stacktrace_parse(const char **input,
         if((frame = sr_koops_frame_parse(&local_input)))
         {
             if (alt_stack)
-                frame->special_stack = sr_strdup(alt_stack);
+                frame->special_stack = g_strdup(alt_stack);
 
             stacktrace->frames = sr_koops_frame_append(stacktrace->frames, frame);
             goto next_line;
@@ -405,6 +408,8 @@ sr_koops_stacktrace_parse(const char **input,
 next_line:
         sr_skip_char(&local_input, '\n');
     }
+    if (alt_stack)
+        g_free(alt_stack);
 
     *input = local_input;
     return stacktrace;
@@ -434,6 +439,13 @@ module_list_continues(const char *input)
         sr_skip_char_span(&input, "0123456789"))
         return false;
 
+    /* CR2 can also be spilled at the end:
+     * https://git.kernel.org/pub/scm/linux/kernel/git/stable/linux.git/tree/arch/x86/mm/fault.c?id=8e998fc24de47c55b47a887f6c95ab91acd4a720#n824
+     */
+    if (sr_skip_string(&input, "CR2: ") &&
+        sr_skip_hexadecimal_uint(&input))
+        return false;
+
     /* Other conditions may need to be added */
 
     return true;
@@ -451,7 +463,7 @@ sr_koops_stacktrace_parse_modules(const char **input)
     int ws = sr_skip_char_span(&local_input, " \t");
 
     int result_size = 20, result_offset = 0;
-    char **result = sr_malloc_array(result_size, sizeof(char*));
+    char **result = g_malloc_n(result_size, sizeof(char*));
 
     char *module;
     while (true)
@@ -463,7 +475,7 @@ sr_koops_stacktrace_parse_modules(const char **input)
             if (result_offset == result_size - 1)
             {
                 result_size *= 2;
-                result = sr_realloc_array(result, result_size, sizeof(char*));
+                result = g_realloc_n(result, result_size, sizeof(char*));
             }
 
             result[result_offset] = module;
@@ -490,8 +502,8 @@ sr_koops_stacktrace_parse_modules(const char **input)
                     break; /* wtf? */
                 }
 
-                char *tmp = sr_asprintf("%s%s", result[result_offset-1], therest);
-                free(result[result_offset-1]);
+                char *tmp = g_strdup_printf("%s%s", result[result_offset-1], therest);
+                g_free(result[result_offset-1]);
                 result[result_offset-1] = tmp;
             }
 
@@ -509,7 +521,7 @@ sr_koops_stacktrace_parse_modules(const char **input)
 static char *
 taint_flags_to_json(struct sr_koops_stacktrace *stacktrace)
 {
-    struct sr_strbuf *strbuf = sr_strbuf_new();
+    GString *strbuf = g_string_new(NULL);
 
     struct sr_taint_flag *f;
     for (f = sr_flags; f->letter; f++)
@@ -517,18 +529,18 @@ taint_flags_to_json(struct sr_koops_stacktrace *stacktrace)
         bool val = *(bool *)((void *)stacktrace + f->member_offset);
         if (val == true)
         {
-            sr_strbuf_append_strf(strbuf, ", \"%s\"\n", f->name);
+            g_string_append_printf(strbuf, ", \"%s\"\n", f->name);
         }
     }
 
     if (strbuf->len == 0)
     {
-        sr_strbuf_free(strbuf);
-        return sr_strdup("[]");
+        g_string_free(strbuf, TRUE);
+        return g_strdup("[]");
     }
 
-    sr_strbuf_append_char(strbuf, ']');
-    char *result = sr_strbuf_free_nobuf(strbuf);
+    g_string_append_c(strbuf, ']');
+    char *result = g_string_free(strbuf, FALSE);
     result[0] = '[';
     result[strlen(result) - 2] = ' '; /* erase the last newline */
     return result;
@@ -537,90 +549,90 @@ taint_flags_to_json(struct sr_koops_stacktrace *stacktrace)
 char *
 sr_koops_stacktrace_to_json(struct sr_koops_stacktrace *stacktrace)
 {
-    struct sr_strbuf *strbuf = sr_strbuf_new();
+    GString *strbuf = g_string_new(NULL);
 
     /* Raw oops. */
     if (stacktrace->raw_oops)
     {
-        sr_strbuf_append_str(strbuf, ",   \"raw_oops\": ");
+        g_string_append(strbuf, ",   \"raw_oops\": ");
         sr_json_append_escaped(strbuf, stacktrace->raw_oops);
-        sr_strbuf_append_str(strbuf, "\n");
+        g_string_append(strbuf, "\n");
     }
 
     /* Kernel version. */
     if (stacktrace->version)
     {
-        sr_strbuf_append_str(strbuf, ",   \"version\": ");
+        g_string_append(strbuf, ",   \"version\": ");
         sr_json_append_escaped(strbuf, stacktrace->version);
-        sr_strbuf_append_str(strbuf, "\n");
+        g_string_append(strbuf, "\n");
     }
 
     /* Kernel taint flags. */
     char *taint_flags = taint_flags_to_json(stacktrace);
     char *indented_taint_flags = sr_indent_except_first_line(taint_flags, strlen(",   \"taint_flags\": "));
-    free(taint_flags);
-    sr_strbuf_append_strf(strbuf, ",   \"taint_flags\": %s\n", indented_taint_flags);
-    free(indented_taint_flags);
+    g_free(taint_flags);
+    g_string_append_printf(strbuf, ",   \"taint_flags\": %s\n", indented_taint_flags);
+    g_free(indented_taint_flags);
 
     /* Modules. */
     if (stacktrace->modules)
     {
-        sr_strbuf_append_strf(strbuf, ",   \"modules\":\n");
-        sr_strbuf_append_str(strbuf, "      [ ");
+        g_string_append_printf(strbuf, ",   \"modules\":\n");
+        g_string_append(strbuf, "      [ ");
 
         char **module = stacktrace->modules;
         while (*module)
         {
             if (module != stacktrace->modules)
-                sr_strbuf_append_str(strbuf, "      , ");
+                g_string_append(strbuf, "      , ");
 
             sr_json_append_escaped(strbuf, *module);
             ++module;
             if (*module)
-                sr_strbuf_append_str(strbuf, "\n");
+                g_string_append(strbuf, "\n");
         }
 
-        sr_strbuf_append_str(strbuf, " ]\n");
+        g_string_append(strbuf, " ]\n");
     }
 
     /* Frames. */
     if (stacktrace->frames)
     {
         struct sr_koops_frame *frame = stacktrace->frames;
-        sr_strbuf_append_str(strbuf, ",   \"frames\":\n");
+        g_string_append(strbuf, ",   \"frames\":\n");
         while (frame)
         {
             if (frame == stacktrace->frames)
-                sr_strbuf_append_str(strbuf, "      [ ");
+                g_string_append(strbuf, "      [ ");
             else
-                sr_strbuf_append_str(strbuf, "      , ");
+                g_string_append(strbuf, "      , ");
 
             char *frame_json = sr_koops_frame_to_json(frame);
             char *indented_frame_json = sr_indent_except_first_line(frame_json, 8);
-            sr_strbuf_append_str(strbuf, indented_frame_json);
-            free(indented_frame_json);
-            free(frame_json);
+            g_string_append(strbuf, indented_frame_json);
+            g_free(indented_frame_json);
+            g_free(frame_json);
             frame = frame->next;
             if (frame)
-                sr_strbuf_append_str(strbuf, "\n");
+                g_string_append(strbuf, "\n");
         }
 
-        sr_strbuf_append_str(strbuf, " ]\n");
+        g_string_append(strbuf, " ]\n");
     }
 
     if (strbuf->len > 0)
-        strbuf->buf[0] = '{';
+        strbuf->str[0] = '{';
     else
-        sr_strbuf_append_char(strbuf, '{');
+        g_string_append_c(strbuf, '{');
 
-    sr_strbuf_append_char(strbuf, '}');
-    return sr_strbuf_free_nobuf(strbuf);
+    g_string_append_c(strbuf, '}');
+    return g_string_free(strbuf, FALSE);
 }
 
 struct sr_koops_stacktrace *
-sr_koops_stacktrace_from_json(struct sr_json_value *root, char **error_message)
+sr_koops_stacktrace_from_json(json_object *root, char **error_message)
 {
-    if (!JSON_CHECK_TYPE(root, SR_JSON_OBJECT, "stacktrace"))
+    if (!json_check_type(root, json_type_object, "stacktrace", error_message))
         return NULL;
 
     struct sr_koops_stacktrace *result = sr_koops_stacktrace_new();
@@ -634,22 +646,34 @@ sr_koops_stacktrace_from_json(struct sr_json_value *root, char **error_message)
         goto fail;
 
     /* Kernel taint flags. */
-    struct sr_json_value *taint_flags = json_element(root, "taint_flags");
-    if (taint_flags)
+    json_object *taint_flags;
+
+    if (json_object_object_get_ex(root, "taint_flags", &taint_flags))
     {
-        if (!JSON_CHECK_TYPE(taint_flags, SR_JSON_ARRAY, "taint_flags"))
+        size_t array_length;
+
+        if (!json_check_type(taint_flags, json_type_array, "taint_flags", error_message))
             goto fail;
 
-        struct sr_json_value *flag_json;
-        FOR_JSON_ARRAY(taint_flags, flag_json)
+        array_length = json_object_array_length(taint_flags);
+
+        for (size_t i = 0; i < array_length; i++)
         {
-            if (!JSON_CHECK_TYPE(flag_json, SR_JSON_STRING, "taint flag"))
+            json_object *flag_json;
+            const char *flag;
+
+            flag_json = json_object_array_get_idx(taint_flags, i);
+
+            if (!json_check_type(flag_json, json_type_string, "taint flag", error_message))
                 goto fail;
 
+            flag = json_object_get_string(flag_json);
+
             for (struct sr_taint_flag *f = sr_flags; f->name; f++)
             {
-                if (0 == strcmp(f->name, flag_json->u.string.ptr))
+                if (0 == strcmp(f->name, flag))
                 {
+                    /* WTF? */
                     *(bool *)((void *)result + f->member_offset) = true;
                     break;
                 }
@@ -659,47 +683,64 @@ sr_koops_stacktrace_from_json(struct sr_json_value *root, char **error_message)
     }
 
     /* Modules. */
-    struct sr_json_value *modules = json_element(root, "modules");
-    if (modules)
+    json_object *modules;
+
+    if (json_object_object_get_ex(root, "modules", &modules))
     {
-        if (!JSON_CHECK_TYPE(modules, SR_JSON_ARRAY, "modules"))
+        size_t array_length;
+        size_t i;
+
+        if (!json_check_type(modules, json_type_array, "modules", error_message))
             goto fail;
 
-        unsigned i = 0;
-        size_t allocated = 128;
-        result->modules = sr_malloc_array(allocated, sizeof(char*));
+        array_length = json_object_array_length(modules);
 
-        struct sr_json_value *mod_json;
-        FOR_JSON_ARRAY(modules, mod_json)
+        size_t allocated = 128;
+        result->modules = g_malloc_n(allocated, sizeof(char*));
+
+        for (i = 0; i < array_length; i++)
         {
-            if (!JSON_CHECK_TYPE(mod_json, SR_JSON_STRING, "module"))
+            json_object *mod_json;
+            const char *module;
+
+            mod_json = json_object_array_get_idx(modules, i);
+
+            if (!json_check_type(mod_json, json_type_string, "module", error_message))
                 goto fail;
 
+            module = json_object_get_string(mod_json);
+
             /* need to keep the last element for NULL terminator */
-            if (i+1 == allocated)
+            if (i + 1 == allocated)
             {
                 allocated *= 2;
-                result->modules = sr_realloc(result->modules, allocated);
+                result->modules = g_realloc(result->modules, allocated);
             }
-            result->modules[i] = sr_strdup(mod_json->u.string.ptr);
-            i++;
+            result->modules[i] = g_strdup(module);
         }
 
         result->modules[i] = NULL;
     }
 
     /* Frames. */
-    struct sr_json_value *frames = json_element(root, "frames");
-    if (frames)
+    json_object *frames;
+
+    if (json_object_object_get_ex(root, "frames", &frames))
     {
-        if (!JSON_CHECK_TYPE(frames, SR_JSON_ARRAY, "frames"))
+        size_t array_length;
+
+        if (!json_check_type(frames, json_type_array, "frames", error_message))
             goto fail;
 
-        struct sr_json_value *frame_json;
-        FOR_JSON_ARRAY(frames, frame_json)
+        array_length = json_object_array_length(frames);
+
+        for (size_t i = 0; i < array_length; i++)
         {
-            struct sr_koops_frame *frame = sr_koops_frame_from_json(frame_json,
-                error_message);
+            json_object *frame_json;
+            struct sr_koops_frame *frame;
+
+            frame_json = json_object_array_get_idx(frames, i);
+            frame = sr_koops_frame_from_json(frame_json, error_message);
 
             if (!frame)
                 goto fail;
@@ -719,7 +760,7 @@ char *
 sr_koops_stacktrace_get_reason(struct sr_koops_stacktrace *stacktrace)
 {
     char *func = "<unknown>";
-    struct sr_strbuf *result = sr_strbuf_new();
+    GString *result = g_string_new(NULL);
 
     struct sr_koops_stacktrace *copy = sr_koops_stacktrace_dup(stacktrace);
     sr_normalize_koops_stacktrace(copy);
@@ -730,50 +771,50 @@ sr_koops_stacktrace_get_reason(struct sr_koops_stacktrace *stacktrace)
     if (stacktrace->reason)
     {
         if (strstr(stacktrace->reason, "general protection fault: "))
-            result = sr_strbuf_append_strf(result, "general protection fault in %s", func);
+            g_string_append_printf(result, "general protection fault in %s", func);
         else if (strstr(stacktrace->reason, "kernel paging request at"))
-            result = sr_strbuf_append_strf(result, "kernel paging request at %s", func);
+            g_string_append_printf(result, "kernel paging request at %s", func);
         else
-            result = sr_strbuf_append_str(result, stacktrace->reason);
+            g_string_append(result, stacktrace->reason);
     }
     else
     {
-        result = sr_strbuf_append_strf(result, "Kernel oops in %s", func);
+        g_string_append_printf(result, "Kernel oops in %s", func);
     }
 
     if (copy->frames && copy->frames->module_name)
-        result = sr_strbuf_append_strf(result, " [%s]", copy->frames->module_name);
+        g_string_append_printf(result, " [%s]", copy->frames->module_name);
 
     sr_koops_stacktrace_free(copy);
 
-    return sr_strbuf_free_nobuf(result);
+    return g_string_free(result, FALSE);
 }
 
 static void
 koops_append_bthash_text(struct sr_koops_stacktrace *stacktrace,
-                         enum sr_bthash_flags flags, struct sr_strbuf *strbuf)
+                         enum sr_bthash_flags flags, GString *strbuf)
 {
-    sr_strbuf_append_strf(strbuf, "Version: %s\n", OR_UNKNOWN(stacktrace->version));
+    g_string_append_printf(strbuf, "Version: %s\n", OR_UNKNOWN(stacktrace->version));
 
-    sr_strbuf_append_str(strbuf, "Flags: ");
+    g_string_append(strbuf, "Flags: ");
     for (struct sr_taint_flag *f = sr_flags; f->letter; f++)
     {
         bool val = *(bool *)((void *)stacktrace + f->member_offset);
         if (val == false)
             continue;
     }
-    sr_strbuf_append_char(strbuf, '\n');
+    g_string_append_c(strbuf, '\n');
 
-    sr_strbuf_append_str(strbuf, "Modules: ");
+    g_string_append(strbuf, "Modules: ");
     for (char **mod = stacktrace->modules; mod && *mod; mod++)
     {
-        sr_strbuf_append_str(strbuf, *mod);
+        g_string_append(strbuf, *mod);
         if (*(mod+1))
-            sr_strbuf_append_str(strbuf, ", ");
+            g_string_append(strbuf, ", ");
     }
-    sr_strbuf_append_char(strbuf, '\n');
+    g_string_append_c(strbuf, '\n');
 
-    sr_strbuf_append_char(strbuf, '\n');
+    g_string_append_c(strbuf, '\n');
 }
 
 void
