@@ -20,7 +20,6 @@
 #include "python/frame.h"
 #include "utils.h"
 #include "location.h"
-#include "strbuf.h"
 #include "json.h"
 #include "generic_frame.h"
 #include "thread.h"
@@ -33,10 +32,10 @@
 
 static void
 python_append_bthash_text(struct sr_python_frame *frame, enum sr_bthash_flags flags,
-                          struct sr_strbuf *strbuf);
+                          GString *strbuf);
 static void
 python_append_duphash_text(struct sr_python_frame *frame, enum sr_duphash_flags flags,
-                           struct sr_strbuf *strbuf);
+                           GString *strbuf);
 
 DEFINE_NEXT_FUNC(python_next, struct sr_frame, struct sr_python_frame)
 DEFINE_SET_NEXT_FUNC(python_set_next, struct sr_frame, struct sr_python_frame)
@@ -61,7 +60,7 @@ struct sr_python_frame *
 sr_python_frame_new()
 {
     struct sr_python_frame *frame =
-        sr_malloc(sizeof(struct sr_python_frame));
+        g_malloc(sizeof(*frame));
 
     sr_python_frame_init(frame);
     return frame;
@@ -80,10 +79,10 @@ sr_python_frame_free(struct sr_python_frame *frame)
     if (!frame)
         return;
 
-    free(frame->file_name);
-    free(frame->function_name);
-    free(frame->line_contents);
-    free(frame);
+    g_free(frame->file_name);
+    g_free(frame->function_name);
+    g_free(frame->line_contents);
+    g_free(frame);
 }
 
 struct sr_python_frame *
@@ -103,13 +102,13 @@ sr_python_frame_dup(struct sr_python_frame *frame, bool siblings)
 
     /* Duplicate all strings. */
     if (result->file_name)
-        result->file_name = sr_strdup(result->file_name);
+        result->file_name = g_strdup(result->file_name);
 
     if (result->function_name)
-        result->function_name = sr_strdup(result->function_name);
+        result->function_name = g_strdup(result->function_name);
 
     if (result->line_contents)
-        result->line_contents = sr_strdup(result->line_contents);
+        result->line_contents = g_strdup(result->line_contents);
 
     return result;
 }
@@ -119,13 +118,13 @@ sr_python_frame_cmp(struct sr_python_frame *frame1,
                     struct sr_python_frame *frame2)
 {
     /* function_name */
-    int function_name = sr_strcmp0(frame1->function_name,
+    int function_name = g_strcmp0(frame1->function_name,
                                    frame2->function_name);
     if (function_name != 0)
         return function_name;
 
     /* file_name */
-    int file_name = sr_strcmp0(frame1->file_name,
+    int file_name = g_strcmp0(frame1->file_name,
                                frame2->file_name);
     if (file_name != 0)
         return file_name;
@@ -148,7 +147,7 @@ sr_python_frame_cmp(struct sr_python_frame *frame1,
         return special_file;
 
     /* line_contents */
-    int line_contents = sr_strcmp0(frame1->line_contents,
+    int line_contents = g_strcmp0(frame1->line_contents,
                                    frame2->line_contents);
     if (line_contents != 0)
         return line_contents;
@@ -161,13 +160,13 @@ sr_python_frame_cmp_distance(struct sr_python_frame *frame1,
                              struct sr_python_frame *frame2)
 {
     /* function_name */
-    int function_name = sr_strcmp0(frame1->function_name,
+    int function_name = g_strcmp0(frame1->function_name,
                                    frame2->function_name);
     if (function_name != 0)
         return function_name;
 
     /* file_name */
-    int file_name = sr_strcmp0(frame1->file_name,
+    int file_name = g_strcmp0(frame1->file_name,
                                frame2->file_name);
     if (file_name != 0)
         return file_name;
@@ -208,9 +207,42 @@ sr_python_frame_parse(const char **input,
 {
     const char *local_input = *input;
 
+    /*
+     * Skip fine-grained error location lines (See: PEP 657),
+     * and syntax error lines. Such lines only contain '^' and '~'
+     * characters.
+     *
+     * Example:
+     *
+     * Traceback (most recent call last):
+     *   File "test.py", line 2, in <module>
+     *     x['a']['b']['c']['d'] = 1
+     *     ~~~~~~~~~~~^^^^^
+     * TypeError: 'NoneType' object is not subscriptable
+     */
+    bool is_error_location_line = true;
+    const char *tmp_input = local_input;
+    while (*tmp_input != '\n' && *tmp_input != '\0')
+    {
+        if (*tmp_input != ' ' && *tmp_input != '^' && *tmp_input != '~')
+        {
+            is_error_location_line = false;
+            break;
+        }
+        ++tmp_input;
+    }
+
+    if (is_error_location_line)
+    {
+        /* Skip the error location line */
+        sr_skip_char_cspan(&local_input, "\n");
+        ++local_input;
+        *input = local_input;
+    }
+
     if (0 == sr_skip_string(&local_input, "  File \""))
     {
-        location->message = sr_asprintf("Frame header not found.");
+        location->message = g_strdup_printf("Frame header not found.");
         return NULL;
     }
 
@@ -220,7 +252,7 @@ sr_python_frame_parse(const char **input,
     /* Parse file name */
     if (!sr_parse_char_cspan(&local_input, "\"", &frame->file_name))
     {
-        location->message = sr_asprintf("Unable to find the '\"' character "
+        location->message = g_strdup_printf("Unable to find the '\"' character "
                 "identifying the beginning of file name.");
 
         goto fail;
@@ -232,8 +264,8 @@ sr_python_frame_parse(const char **input,
     {
         frame->special_file = true;
         frame->file_name[strlen(frame->file_name)-1] = '\0';
-        char *inside = sr_strdup(frame->file_name + 1);
-        free(frame->file_name);
+        char *inside = g_strdup(frame->file_name + 1);
+        g_free(frame->file_name);
         frame->file_name = inside;
     }
 
@@ -243,7 +275,7 @@ sr_python_frame_parse(const char **input,
 
     if (0 == sr_skip_string(&local_input, "\", line "))
     {
-        location->message = sr_asprintf("Line separator not found.");
+        location->message = g_strdup_printf("Line separator not found.");
         goto fail;
     }
 
@@ -253,7 +285,7 @@ sr_python_frame_parse(const char **input,
     int length = sr_parse_uint32(&local_input, &frame->file_line);
     if (0 == length)
     {
-        location->message = sr_asprintf("Line number not found.");
+        location->message = g_strdup_printf("Line number not found.");
         goto fail;
     }
 
@@ -263,7 +295,7 @@ sr_python_frame_parse(const char **input,
     {
         if (local_input[0] != '\n')
         {
-            location->message = sr_asprintf("Function name separator not found.");
+            location->message = g_strdup_printf("Function name separator not found.");
             goto fail;
         }
 
@@ -271,7 +303,7 @@ sr_python_frame_parse(const char **input,
          * function name on its line. For the sake of simplicity, we will
          * believe that we are dealing with such a frame now.
          */
-        frame->function_name = sr_strdup("syntax");
+        frame->function_name = g_strdup("syntax");
         frame->special_function = true;
     }
     else
@@ -281,7 +313,7 @@ sr_python_frame_parse(const char **input,
         /* Parse function name */
         if (!sr_parse_char_cspan(&local_input, "\n", &frame->function_name))
         {
-            location->message = sr_asprintf("Unable to find the newline character "
+            location->message = g_strdup_printf("Unable to find the newline character "
                     "identifying the end of function name.");
 
             goto fail;
@@ -295,8 +327,8 @@ sr_python_frame_parse(const char **input,
         {
             frame->special_function = true;
             frame->function_name[strlen(frame->function_name)-1] = '\0';
-            char *inside = sr_strdup(frame->function_name + 1);
-            free(frame->function_name);
+            char *inside = g_strdup(frame->function_name + 1);
+            g_free(frame->function_name);
             frame->function_name = inside;
         }
     }
@@ -323,24 +355,24 @@ fail:
 char *
 sr_python_frame_to_json(struct sr_python_frame *frame)
 {
-    struct sr_strbuf *strbuf = sr_strbuf_new();
+    GString *strbuf = g_string_new(NULL);
 
     /* Source file name / special file. */
     if (frame->file_name)
     {
         if (frame->special_file)
-            sr_strbuf_append_str(strbuf, ",   \"special_file\": ");
+            g_string_append(strbuf, ",   \"special_file\": ");
         else
-            sr_strbuf_append_str(strbuf, ",   \"file_name\": ");
+            g_string_append(strbuf, ",   \"file_name\": ");
 
         sr_json_append_escaped(strbuf, frame->file_name);
-        sr_strbuf_append_str(strbuf, "\n");
+        g_string_append(strbuf, "\n");
     }
 
     /* Source file line. */
     if (frame->file_line)
     {
-        sr_strbuf_append_strf(strbuf,
+        g_string_append_printf(strbuf,
                               ",   \"file_line\": %"PRIu32"\n",
                               frame->file_line);
     }
@@ -349,70 +381,86 @@ sr_python_frame_to_json(struct sr_python_frame *frame)
     if (frame->function_name)
     {
         if (frame->special_function)
-            sr_strbuf_append_str(strbuf, ",   \"special_function\": ");
+            g_string_append(strbuf, ",   \"special_function\": ");
         else
-            sr_strbuf_append_str(strbuf, ",   \"function_name\": ");
+            g_string_append(strbuf, ",   \"function_name\": ");
 
         sr_json_append_escaped(strbuf, frame->function_name);
-        sr_strbuf_append_str(strbuf, "\n");
+        g_string_append(strbuf, "\n");
     }
 
     /* Line contents. */
     if (frame->line_contents)
     {
-        sr_strbuf_append_str(strbuf, ",   \"line_contents\": ");
+        g_string_append(strbuf, ",   \"line_contents\": ");
         sr_json_append_escaped(strbuf, frame->line_contents);
-        sr_strbuf_append_str(strbuf, "\n");
+        g_string_append(strbuf, "\n");
     }
 
-    strbuf->buf[0] = '{';
-    sr_strbuf_append_char(strbuf, '}');
-    return sr_strbuf_free_nobuf(strbuf);
+    strbuf->str[0] = '{';
+    g_string_append_c(strbuf, '}');
+    return g_string_free(strbuf, FALSE);
 }
 
 struct sr_python_frame *
-sr_python_frame_from_json(struct sr_json_value *root, char **error_message)
+sr_python_frame_from_json(json_object *root, char **error_message)
 {
-    if (!JSON_CHECK_TYPE(root, SR_JSON_OBJECT, "frame"))
+    if (!json_check_type(root, json_type_object, "frame", error_message))
         return NULL;
 
     struct sr_python_frame *result = sr_python_frame_new();
-    struct sr_json_value *val;
+    json_object *val;
 
     /* Source file name / special file */
-    if ((val = json_element(root, "file_name")))
+    if (json_object_object_get_ex(root, "file_name", &val))
     {
-        if (!JSON_CHECK_TYPE(val, SR_JSON_STRING, "file_name"))
+        const char *string;
+
+        if (!json_check_type(val, json_type_string, "file_name", error_message))
             goto fail;
 
+        string = json_object_get_string(val);
+
         result->special_file = false;
-        result->file_name = sr_strdup(val->u.string.ptr);
+        result->file_name = g_strdup(string);
     }
-    else if ((val = json_element(root, "special_file")))
+    else if (json_object_object_get_ex(root, "special_file", &val))
     {
-        if (!JSON_CHECK_TYPE(val, SR_JSON_STRING, "special_file"))
+        const char *string;
+
+        if (!json_check_type(val, json_type_string, "special_file", error_message))
             goto fail;
 
+        string = json_object_get_string(val);
+
         result->special_file = true;
-        result->file_name = sr_strdup(val->u.string.ptr);
+        result->file_name = g_strdup(string);
     }
 
     /* Function name / special function. */
-    if ((val = json_element(root, "function_name")))
+    if (json_object_object_get_ex(root, "function_name", &val))
     {
-        if (!JSON_CHECK_TYPE(val, SR_JSON_STRING, "function_name"))
+        const char *string;
+
+        if (!json_check_type(val, json_type_string, "function_name", error_message))
             goto fail;
 
+        string = json_object_get_string(val);
+
         result->special_function = false;
-        result->function_name = sr_strdup(val->u.string.ptr);
+        result->function_name = g_strdup(string);
     }
-    else if ((val = json_element(root, "special_function")))
+    else if (json_object_object_get_ex(root, "special_function", &val))
     {
-        if (!JSON_CHECK_TYPE(val, SR_JSON_STRING, "special_function"))
+        const char *string;
+
+        if (!json_check_type(val, json_type_string, "special_function", error_message))
             goto fail;
 
+        string = json_object_get_string(val);
+
         result->special_function = true;
-        result->function_name = sr_strdup(val->u.string.ptr);
+        result->function_name = g_strdup(string);
     }
 
     bool success =
@@ -431,24 +479,24 @@ fail:
 
 void
 sr_python_frame_append_to_str(struct sr_python_frame *frame,
-                              struct sr_strbuf *dest)
+                              GString *dest)
 {
     if (frame->file_name)
     {
-        sr_strbuf_append_strf(dest, "[%s%s%s",
+        g_string_append_printf(dest, "[%s%s%s",
                               (frame->special_file ? "<" : ""),
                               frame->file_name,
                               (frame->special_file ? ">" : ""));
 
         if (frame->file_line)
         {
-            sr_strbuf_append_strf(dest, ":%"PRIu32, frame->file_line);
+            g_string_append_printf(dest, ":%"PRIu32, frame->file_line);
         }
 
-        sr_strbuf_append_str(dest, "]");
+        g_string_append(dest, "]");
     }
 
-    sr_strbuf_append_strf(dest, " %s%s%s",
+    g_string_append_printf(dest, " %s%s%s",
                           (frame->special_function ? "<" : ""),
                           (frame->function_name ? frame->function_name : "??"),
                           (frame->special_function ? ">" : ""));
@@ -456,9 +504,9 @@ sr_python_frame_append_to_str(struct sr_python_frame *frame,
 
 static void
 python_append_bthash_text(struct sr_python_frame *frame, enum sr_bthash_flags flags,
-                          struct sr_strbuf *strbuf)
+                          GString *strbuf)
 {
-    sr_strbuf_append_strf(strbuf,
+    g_string_append_printf(strbuf,
                           "%s, %d, %"PRIu32", %s, %d, %s\n",
                           OR_UNKNOWN(frame->file_name),
                           frame->special_file,
@@ -470,10 +518,10 @@ python_append_bthash_text(struct sr_python_frame *frame, enum sr_bthash_flags fl
 
 static void
 python_append_duphash_text(struct sr_python_frame *frame, enum sr_duphash_flags flags,
-                          struct sr_strbuf *strbuf)
+                          GString *strbuf)
 {
     /* filename:line */
-    sr_strbuf_append_strf(strbuf, "%s:%"PRIu32"\n",
+    g_string_append_printf(strbuf, "%s:%"PRIu32"\n",
                           OR_UNKNOWN(frame->file_name),
                           frame->file_line);
 }
