@@ -20,7 +20,6 @@
 #include "rpm.h"
 #include "utils.h"
 #include "json.h"
-#include "strbuf.h"
 #include "config.h"
 #include "internal_utils.h"
 #include <errno.h>
@@ -39,7 +38,7 @@ struct sr_rpm_package *
 sr_rpm_package_new()
 {
     struct sr_rpm_package *package =
-        sr_malloc(sizeof(struct sr_rpm_package));
+        g_malloc(sizeof(*package));
 
     sr_rpm_package_init(package);
     return package;
@@ -58,15 +57,15 @@ sr_rpm_package_free(struct sr_rpm_package *package,
     if (!package)
         return;
 
-    free(package->name);
-    free(package->version);
-    free(package->release);
-    free(package->architecture);
+    g_free(package->name);
+    g_free(package->version);
+    g_free(package->release);
+    g_free(package->architecture);
     sr_rpm_consistency_free(package->consistency, true);
     if (package->next && recursive)
         sr_rpm_package_free(package->next, true);
 
-    free(package);
+    g_free(package);
 }
 
 int
@@ -106,7 +105,7 @@ sr_rpm_package_cmp_nevra(struct sr_rpm_package *package1,
         return epoch;
 
     /* Architecture. */
-    int architecture = sr_strcmp0(package1->architecture,
+    int architecture = g_strcmp0(package1->architecture,
                                    package2->architecture);
     if (architecture != 0)
         return architecture;
@@ -119,17 +118,17 @@ sr_rpm_package_cmp_nvr(struct sr_rpm_package *package1,
                        struct sr_rpm_package *package2)
 {
     /* Name. */
-    int name = sr_strcmp0(package1->name, package2->name);
+    int name = g_strcmp0(package1->name, package2->name);
     if (name != 0)
         return name;
 
     /* Version. */
-    int version = sr_strcmp0(package1->version, package2->version);
+    int version = g_strcmp0(package1->version, package2->version);
     if (version != 0)
         return version;
 
     /* Release. */
-    int release = sr_strcmp0(package1->release, package2->release);
+    int release = g_strcmp0(package1->release, package2->release);
     if (release != 0)
         return release;
 
@@ -176,7 +175,7 @@ struct sr_rpm_package *
 sr_rpm_package_sort(struct sr_rpm_package *packages)
 {
     size_t count = sr_rpm_package_count(packages);
-    struct sr_rpm_package **array = sr_malloc_array(count, sizeof(struct sr_rpm_package*));
+    struct sr_rpm_package **array = g_malloc_n(count, sizeof(struct sr_rpm_package*));
 
     /* Copy the linked list to an array. */
     struct sr_rpm_package *list_loop = packages, **array_loop = array;
@@ -200,7 +199,7 @@ sr_rpm_package_sort(struct sr_rpm_package *packages)
     }
 
     struct sr_rpm_package *result = array[0];
-    free(array);
+    g_free(array);
     return result;
 }
 
@@ -214,20 +213,20 @@ package_merge(struct sr_rpm_package *p1, struct sr_rpm_package *p2)
         return NULL;
 
     if (p1->architecture && p2->architecture &&
-        0 != sr_strcmp0(p1->architecture, p2->architecture))
+        0 != g_strcmp0(p1->architecture, p2->architecture))
         return NULL;
 
     struct sr_rpm_package *merged = sr_rpm_package_new();
-    merged->name = sr_strdup(p1->name);
+    merged->name = g_strdup(p1->name);
     merged->epoch = p1->epoch;
-    merged->version = sr_strdup(p1->version);
-    merged->release = sr_strdup(p1->release);
+    merged->version = g_strdup(p1->version);
+    merged->release = g_strdup(p1->release);
 
     /* architecture is sometimes missing */
     if (p1->architecture)
-        merged->architecture = sr_strdup(p1->architecture);
+        merged->architecture = g_strdup(p1->architecture);
     else if (p2->architecture)
-        merged->architecture = sr_strdup(p2->architecture);
+        merged->architecture = g_strdup(p2->architecture);
 
     merged->install_time = (p1->install_time ?
                             p1->install_time : p2->install_time);
@@ -289,7 +288,7 @@ header_get_string(Header header,
         return false;
 
     const char *str = rpmtdGetString(tag_data);
-    *result = (str ? sr_strdup(str) : NULL);
+    *result = (str ? g_strdup(str) : NULL);
     rpmtdFree(tag_data);
     return str;
 }
@@ -325,14 +324,14 @@ header_to_rpm_info(Header header,
     if (!header_get_string(header, RPMTAG_NAME, &rpm->name))
     {
         sr_rpm_package_free(rpm, false);
-        *error_message = sr_asprintf("Failed to find package name.");
+        *error_message = g_strdup_printf("Failed to find package name.");
         return NULL;
     }
 
     bool success = header_get_uint32(header, RPMTAG_EPOCH, &rpm->epoch);
     if (!success)
     {
-        *error_message = sr_asprintf("Failed to find package epoch.");
+        *error_message = g_strdup_printf("Failed to find package epoch.");
         sr_rpm_package_free(rpm, false);
         return NULL;
     }
@@ -340,7 +339,7 @@ header_to_rpm_info(Header header,
     success = header_get_string(header, RPMTAG_VERSION, &rpm->version);
     if (!success)
     {
-        *error_message = sr_asprintf("Failed to find package version.");
+        *error_message = g_strdup_printf("Failed to find package version.");
         sr_rpm_package_free(rpm, false);
         return NULL;
     }
@@ -348,7 +347,7 @@ header_to_rpm_info(Header header,
     success = header_get_string(header, RPMTAG_RELEASE, &rpm->release);
     if (!success)
     {
-        *error_message = sr_asprintf("Failed to find package release.");
+        *error_message = g_strdup_printf("Failed to find package release.");
         sr_rpm_package_free(rpm, false);
         return NULL;
     }
@@ -356,7 +355,7 @@ header_to_rpm_info(Header header,
     success = header_get_string(header, RPMTAG_ARCH, &rpm->architecture);
     if (!success)
     {
-        *error_message = sr_asprintf("Failed to find package architecture.");
+        *error_message = g_strdup_printf("Failed to find package architecture.");
         sr_rpm_package_free(rpm, false);
         return NULL;
     }
@@ -365,7 +364,7 @@ header_to_rpm_info(Header header,
     success = header_get_uint32(header, RPMTAG_INSTALLTIME, &install_time);
     if (!success)
     {
-        *error_message = sr_asprintf("Failed to find package installation time.");
+        *error_message = g_strdup_printf("Failed to find package installation time.");
         sr_rpm_package_free(rpm, false);
         return NULL;
     }
@@ -386,7 +385,7 @@ sr_rpm_package_get_by_name(const char *name, char **error_message)
 #ifdef HAVE_LIBRPM
     if (rpmReadConfigFiles(NULL, NULL))
     {
-        *error_message = sr_asprintf("Failed to read RPM configuration files.");
+        *error_message = g_strdup_printf("Failed to read RPM configuration files.");
         return NULL;
     }
 
@@ -416,7 +415,7 @@ sr_rpm_package_get_by_name(const char *name, char **error_message)
     rpmtsFree(ts);
     return result;
 #else
-    *error_message = sr_asprintf("satyr compiled without rpm");
+    *error_message = g_strdup_printf("satyr compiled without rpm");
     return NULL;
 #endif
 }
@@ -428,7 +427,7 @@ sr_rpm_package_get_by_path(const char *path,
 #ifdef HAVE_LIBRPM
     if (rpmReadConfigFiles(NULL, NULL))
     {
-        *error_message = sr_asprintf("Failed to read RPM configuration files.");
+        *error_message = g_strdup_printf("Failed to read RPM configuration files.");
         return NULL;
     }
 
@@ -458,7 +457,7 @@ sr_rpm_package_get_by_path(const char *path,
     rpmtsFree(ts);
     return result;
 #else
-    *error_message = sr_asprintf("satyr compiled without rpm");
+    *error_message = g_strdup_printf("satyr compiled without rpm");
     return NULL;
 #endif
 }
@@ -467,7 +466,7 @@ char *
 sr_rpm_package_to_json(struct sr_rpm_package *package,
                        bool recursive)
 {
-    struct sr_strbuf *strbuf = sr_strbuf_new();
+    GString *strbuf = g_string_new(NULL);
 
     if (recursive)
     {
@@ -475,65 +474,65 @@ sr_rpm_package_to_json(struct sr_rpm_package *package,
         while (p)
         {
             if (p == package)
-                sr_strbuf_append_str(strbuf, "[ ");
+                g_string_append(strbuf, "[ ");
             else
-                sr_strbuf_append_str(strbuf, ", ");
+                g_string_append(strbuf, ", ");
 
             char *package_json = sr_rpm_package_to_json(p, false);
             char *indented_package_json = sr_indent_except_first_line(package_json, 2);
-            sr_strbuf_append_str(strbuf, indented_package_json);
-            free(indented_package_json);
-            free(package_json);
+            g_string_append(strbuf, indented_package_json);
+            g_free(indented_package_json);
+            g_free(package_json);
             p = p->next;
             if (p)
-                sr_strbuf_append_str(strbuf, "\n");
+                g_string_append(strbuf, "\n");
         }
 
-        sr_strbuf_append_str(strbuf, " ]");
+        g_string_append(strbuf, " ]");
     }
     else
     {
         /* Name. */
         if (package->name)
         {
-            sr_strbuf_append_str(strbuf, ",   \"name\": ");
+            g_string_append(strbuf, ",   \"name\": ");
             sr_json_append_escaped(strbuf, package->name);
-            sr_strbuf_append_str(strbuf, "\n");
+            g_string_append(strbuf, "\n");
         }
 
         /* Epoch. */
-        sr_strbuf_append_strf(strbuf,
+        g_string_append_printf(strbuf,
                               ",   \"epoch\": %"PRIu32"\n",
                               package->epoch);
 
         /* Version. */
         if (package->version)
         {
-            sr_strbuf_append_str(strbuf, ",   \"version\": ");
+            g_string_append(strbuf, ",   \"version\": ");
             sr_json_append_escaped(strbuf, package->version);
-            sr_strbuf_append_str(strbuf, "\n");
+            g_string_append(strbuf, "\n");
         }
 
         /* Release. */
         if (package->release)
         {
-            sr_strbuf_append_str(strbuf, ",   \"release\": ");
+            g_string_append(strbuf, ",   \"release\": ");
             sr_json_append_escaped(strbuf, package->release);
-            sr_strbuf_append_str(strbuf, "\n");
+            g_string_append(strbuf, "\n");
         }
 
         /* Architecture. */
         if (package->architecture)
         {
-            sr_strbuf_append_str(strbuf, ",   \"architecture\": ");
+            g_string_append(strbuf, ",   \"architecture\": ");
             sr_json_append_escaped(strbuf, package->architecture);
-            sr_strbuf_append_str(strbuf, "\n");
+            g_string_append(strbuf, "\n");
         }
 
         /* Install time. */
         if (package->install_time > 0)
         {
-            sr_strbuf_append_strf(strbuf,
+            g_string_append_printf(strbuf,
                                   ",   \"install_time\": %"PRIu64"\n",
                                   package->install_time);
         }
@@ -552,29 +551,29 @@ sr_rpm_package_to_json(struct sr_rpm_package *package,
                 break;
             }
 
-            sr_strbuf_append_strf(strbuf, ",   \"package_role\": \"%s\"\n", role);
+            g_string_append_printf(strbuf, ",   \"package_role\": \"%s\"\n", role);
         }
 
         /* Consistency. */
         if (package->consistency)
         {
             // TODO
-            //sr_strbuf_append_strf(strbuf,
+            //g_string_append_printf(strbuf,
             //                      ",   \"consistency\": \"%s\"\n",
             //                      package->architecture);
         }
 
-        strbuf->buf[0] = '{';
-        sr_strbuf_append_str(strbuf, "}");
+        strbuf->str[0] = '{';
+        g_string_append(strbuf, "}");
     }
 
-    return sr_strbuf_free_nobuf(strbuf);
+    return g_string_free(strbuf, FALSE);
 }
 
 static struct sr_rpm_package *
-single_rpm_pacakge_from_json(struct sr_json_value *root, char **error_message)
+single_rpm_package_from_json(json_object *root, char **error_message)
 {
-    if (!JSON_CHECK_TYPE(root, SR_JSON_OBJECT, "package"))
+    if (!json_check_type(root, json_type_object, "package", error_message))
         return NULL;
 
     struct sr_rpm_package *package = sr_rpm_package_new();
@@ -590,18 +589,22 @@ single_rpm_pacakge_from_json(struct sr_json_value *root, char **error_message)
     if (!success)
         goto fail;
 
-    struct sr_json_value *role_json = json_element(root, "package_role");
-    if (role_json)
+    json_object *role_json;
+
+    if (json_object_object_get_ex(root, "package_role", &role_json))
     {
-        if (!JSON_CHECK_TYPE(role_json, SR_JSON_STRING, "package_role"))
+        if (!json_check_type(role_json, json_type_string, "package_role", error_message))
             goto fail;
 
         /* We only know "affected" so far. */
-        char *role = role_json->u.string.ptr;
+        const char *role;
+
+        role = json_object_get_string(role_json);
+
         if (0 != strcmp(role, "affected"))
         {
             if (error_message)
-                *error_message = sr_asprintf("Invalid package role %s", role);
+                *error_message = g_strdup_printf("Invalid package role %s", role);
             goto fail;
         }
 
@@ -616,24 +619,31 @@ fail:
 }
 
 int
-sr_rpm_package_from_json(struct sr_rpm_package **rpm_package, struct sr_json_value *json,
+sr_rpm_package_from_json(struct sr_rpm_package **rpm_package, json_object *json,
                          bool recursive, char **error_message)
 {
     if (!recursive)
     {
-        *rpm_package = single_rpm_pacakge_from_json(json, error_message);
+        *rpm_package = single_rpm_package_from_json(json, error_message);
         return 0;
     }
     else
     {
-        if (!JSON_CHECK_TYPE(json, SR_JSON_ARRAY, "package list"))
+        size_t array_length;
+
+        if (!json_check_type(json, json_type_array, "package list", error_message))
             return -1;
         struct sr_rpm_package *result = NULL;
 
-        struct sr_json_value *pkg_json;
-        FOR_JSON_ARRAY(json, pkg_json)
+        array_length = json_object_array_length(json);
+
+        for (size_t i = 0; i < array_length; i++)
         {
-            struct sr_rpm_package *pkg = single_rpm_pacakge_from_json(pkg_json, error_message);
+            json_object *pkg_json;
+            struct sr_rpm_package *pkg;
+
+            pkg_json = json_object_array_get_idx(json, i);
+            pkg = single_rpm_package_from_json(pkg_json, error_message);
             if (!pkg)
                 goto fail;
 
@@ -677,14 +687,14 @@ sr_rpm_package_parse_nvr(const char *text,
     }
 
     // Release is after the last dash.
-    *release = sr_strdup(last_dash + 1);
+    *release = g_strdup(last_dash + 1);
 
     // Version is before the last dash.
-    *version = sr_strndup(last_but_one_dash + 1,
+    *version = g_strndup(last_but_one_dash + 1,
                           last_dash - last_but_one_dash - 1);
 
     // Name is before version.
-    *name = sr_strndup(text, last_but_one_dash - text);
+    *name = g_strndup(text, last_but_one_dash - text);
 
     return true;
 }
@@ -730,19 +740,19 @@ sr_rpm_package_parse_nevra(const char *text,
     }
 
     // Architecture is after the last dot.
-    *architecture = sr_strdup(last_dot + 1);
+    *architecture = g_strdup(last_dot + 1);
 
     // Release is after the last dash.
-    *release = sr_strndup(last_dash + 1, last_dot - last_dash - 1);
+    *release = g_strndup(last_dash + 1, last_dot - last_dash - 1);
 
     // Epoch is optional.
     if (colon)
     {
         // Version is before the last dash.
-        *version = sr_strndup(colon + 1,
+        *version = g_strndup(colon + 1,
                               last_dash - colon - 1);
 
-        char *epoch_str = sr_strndup(last_but_one_dash + 1,
+        char *epoch_str = g_strndup(last_but_one_dash + 1,
                                      colon - last_but_one_dash - 1);
 
         char *endptr;
@@ -753,26 +763,26 @@ sr_rpm_package_parse_nevra(const char *text,
 
         if (failure)
         {
-            free(epoch_str);
-            free(*release);
-            free(*architecture);
+            g_free(epoch_str);
+            g_free(*release);
+            g_free(*architecture);
             return false;
         }
 
         *epoch = r;
-        free(epoch_str);
+        g_free(epoch_str);
     }
     else
     {
         // Version is before the last dash.
-        *version = sr_strndup(last_but_one_dash + 1,
+        *version = g_strndup(last_but_one_dash + 1,
                               last_dash - last_but_one_dash - 1);
 
         *epoch = 0;
     }
 
     // Name is before version.
-    *name = sr_strndup(text, last_but_one_dash - text);
+    *name = g_strndup(text, last_but_one_dash - text);
 
     return true;
 }
@@ -781,7 +791,7 @@ struct sr_rpm_consistency *
 sr_rpm_consistency_new()
 {
     struct sr_rpm_consistency *consistency =
-        sr_malloc(sizeof(struct sr_rpm_consistency));
+        g_malloc(sizeof(*consistency));
 
     sr_rpm_consistency_init(consistency);
     return consistency;
@@ -800,11 +810,11 @@ sr_rpm_consistency_free(struct sr_rpm_consistency *consistency,
     if (!consistency)
         return;
 
-    free(consistency->path);
+    g_free(consistency->path);
     if (consistency->next && recursive)
         sr_rpm_consistency_free(consistency->next, true);
 
-    free(consistency);
+    g_free(consistency);
 }
 
 int
@@ -812,7 +822,7 @@ sr_rpm_consistency_cmp(struct sr_rpm_consistency *consistency1,
                        struct sr_rpm_consistency *consistency2)
 {
     /* Path. */
-    int path = sr_strcmp0(consistency1->path, consistency2->path);
+    int path = g_strcmp0(consistency1->path, consistency2->path);
     if (path != 0)
         return path;
 
