@@ -19,7 +19,6 @@
 */
 #include "utils.h"
 #include "location.h"
-#include "strbuf.h"
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -30,9 +29,11 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <ctype.h>
+#include <glib.h>
 
 #define ANONYMIZED_PATH "/home/anonymized"
-
+/* Maximum allowed size of a file when reading into memory. Currently 20 MB. */
+#define FILE_SIZE_LIMIT 20000000L
 
 /* The prototype is in C++ header cxxabi.h, let's just copypaste it here
  * instead of fiddling with include directories */
@@ -60,123 +61,6 @@ warn(const char *fmt, ...)
 
 }
 
-void *
-sr_malloc(size_t size)
-{
-    void *ptr = malloc(size);
-    if (!ptr)
-    {
-        fprintf(stderr, "sr_malloc: out of memory");
-        exit(1);
-    }
-    return ptr;
-}
-
-static void
-check_overflow_size_t(size_t a, size_t b)
-{
-    if (b && a > SIZE_MAX / b)
-    {
-        fprintf(stderr, "sr_{m,re}alloc_array: unsigned overflow detected");
-        exit(1);
-    }
-}
-
-void *
-sr_malloc_array(size_t elems, size_t elem_size)
-{
-    check_overflow_size_t(elems, elem_size);
-    return sr_malloc(elems * elem_size);
-}
-
-void *
-sr_mallocz(size_t size)
-{
-    void *ptr = sr_malloc(size);
-    memset(ptr, 0, size);
-    return ptr;
-}
-
-void *
-sr_realloc(void *ptr, size_t size)
-{
-    void *result = realloc(ptr, size);
-    /* When size is 0, realloc may return NULL on success. */
-    if (!result && size > 0)
-    {
-        fprintf(stderr, "sr_realloc: out of memory");
-        exit(1);
-    }
-    return result;
-}
-
-void *
-sr_realloc_array(void *ptr, size_t elems, size_t elem_size)
-{
-    check_overflow_size_t(elems, elem_size);
-    return sr_realloc(ptr, elems * elem_size);
-}
-
-char *
-sr_vasprintf(const char *format, va_list p)
-{
-    int r;
-    char *string_ptr;
-
-#if 1
-    // GNU extension
-    r = vasprintf(&string_ptr, format, p);
-#else
-    // Bloat for systems that haven't got the GNU extension.
-    va_list p2;
-    va_copy(p2, p);
-    r = vsnprintf(NULL, 0, format, p);
-    string_ptr = sr_malloc(r+1);
-    r = vsnprintf(string_ptr, r+1, format, p2);
-    va_end(p2);
-#endif
-
-    if (r < 0)
-    {
-        fprintf(stderr, "satyr: out of memory");
-        exit(1);
-    }
-
-    return string_ptr;
-}
-
-char *
-sr_asprintf(const char *format, ...)
-{
-    va_list p;
-    char *string_ptr;
-
-    va_start(p, format);
-    string_ptr = sr_vasprintf(format, p);
-    va_end(p);
-
-    return string_ptr;
-}
-
-
-char *
-sr_strdup(const char *s)
-{
-    return sr_strndup(s, strlen(s));
-}
-
-char *
-sr_strndup(const char *s, size_t n)
-{
-    char *result = strndup(s, n);
-    if (result == NULL)
-    {
-        fprintf(stderr, "satyr: out of memory");
-        exit(1);
-    }
-    return result;
-}
-
 void
 sr_struniq(char **strings, size_t *size)
 {
@@ -195,19 +79,10 @@ sr_struniq(char **strings, size_t *size)
     }
 }
 
-int
-sr_strcmp0(const char *s1, const char *s2)
-{
-    if (s1)
-        return s2 ? strcmp(s1, s2) : 1;
-    else
-        return s2 ? -1 : 0;
-}
-
 int
 sr_ptrstrcmp(const void *s1, const void *s2)
 {
-    return sr_strcmp0(*(const char**)s1, *(const char**)s2);
+    return g_strcmp0(*(const char**)s1, *(const char**)s2);
 }
 
 char *
@@ -282,7 +157,7 @@ sr_file_to_string(const char *filename,
     int fd = open(filename, O_RDONLY | O_LARGEFILE);
     if (fd < 0)
     {
-        *error_message = sr_asprintf("Unable to open '%s': %s.",
+        *error_message = g_strdup_printf("Unable to open '%s': %s.",
                                      filename,
                                      strerror(errno));
 
@@ -292,7 +167,7 @@ sr_file_to_string(const char *filename,
     off_t size = lseek(fd, 0, SEEK_END);
     if (size == (off_t)-1) /* EOVERFLOW? */
     {
-        *error_message = sr_asprintf("Unable to seek in '%s': %s.",
+        *error_message = g_strdup_printf("Unable to seek in '%s': %s.",
                                      filename,
                                      strerror(errno));
 
@@ -302,10 +177,9 @@ sr_file_to_string(const char *filename,
 
     lseek(fd, 0, SEEK_SET); /* No reason to fail. */
 
-    static const size_t FILE_SIZE_LIMIT = 20000000; /* ~ 20 MB */
     if (size > FILE_SIZE_LIMIT)
     {
-        *error_message = sr_asprintf("Input file too big (%lld). Maximum size is %zu.",
+        *error_message = g_strdup_printf("Input file too big (%lld). Maximum size is %ld.",
                                      (long long)size,
                                      FILE_SIZE_LIMIT);
 
@@ -313,12 +187,12 @@ sr_file_to_string(const char *filename,
         return NULL;
     }
 
-    char *contents = sr_malloc(size + 1);
+    char *contents = g_malloc(size + 1);
     if (size != read(fd, contents, size))
     {
-        *error_message = sr_asprintf("Unable to read from '%s'.", filename);
+        *error_message = g_strdup_printf("Unable to read from '%s'.", filename);
         close(fd);
-        free(contents);
+        g_free(contents);
         return NULL;
     }
 
@@ -338,7 +212,7 @@ sr_string_to_file(const char *filename,
     int fd = open(filename, O_WRONLY | O_LARGEFILE | O_TRUNC | O_CREAT, 0640);
     if (fd < 0)
     {
-        *error_message = sr_asprintf("Unable to open '%s': %s.",
+        *error_message = g_strdup_printf("Unable to open '%s': %s.",
                                      filename,
                                      strerror(errno));
 
@@ -353,7 +227,7 @@ sr_string_to_file(const char *filename,
         if (-1 == write_result)
             error = strerror(errno);
 
-        *error_message = sr_asprintf("Unable to write to '%s': %s.",
+        *error_message = g_strdup_printf("Unable to write to '%s': %s.",
                                      filename,
                                      error);
 
@@ -364,7 +238,7 @@ sr_string_to_file(const char *filename,
     int close_result = close(fd);
     if (-1 == close_result)
     {
-        *error_message = sr_asprintf("Unable to close '%s': %s.",
+        *error_message = g_strdup_printf("Unable to close '%s': %s.",
                                      filename,
                                      strerror(errno));
 
@@ -445,7 +319,7 @@ sr_parse_char_span(const char **input, const char *accept, char **result)
     size_t count = strspn(*input, accept);
     if (count == 0)
         return 0;
-    *result = sr_strndup(*input, count);
+    *result = g_strndup(*input, count);
     *input += count;
     return count;
 }
@@ -466,7 +340,7 @@ sr_parse_char_cspan(const char **input, const char *reject, char **result)
     size_t count = strcspn(*input, reject);
     if (count == 0)
         return false;
-    *result = sr_strndup(*input, count);
+    *result = g_strndup(*input, count);
     *input += count;
     return true;
 }
@@ -500,7 +374,7 @@ sr_parse_string(const char **input, const char *string, char **result)
     }
     if (*local_string != '\0')
         return false;
-    *result = sr_strndup(string, local_input - *input);
+    *result = g_strndup(string, local_input - *input);
     *input = local_input;
     return true;
 }
@@ -539,7 +413,7 @@ sr_parse_uint32(const char **input, uint32_t *result)
     bool failure = (errno || numstr == endptr || *endptr != '\0'
                     || r > UINT32_MAX);
 
-    free(numstr);
+    g_free(numstr);
     if (failure) /* number too big or some other error */
         return 0;
 
@@ -564,7 +438,7 @@ sr_parse_uint64(const char **input, uint64_t *result)
     unsigned long long result_tmp = strtoull(numstr, &endptr, 10);
     bool failure = (errno || numstr == endptr || *endptr != '\0'
                     || result_tmp == UINT64_MAX);
-    free(numstr);
+    g_free(numstr);
     if (failure) /* number too big or some other error */
         return 0;
     *result = result_tmp;
@@ -613,7 +487,7 @@ sr_parse_hexadecimal_uint64(const char **input, uint64_t *result)
     errno = 0;
     unsigned long long r = strtoull(numstr, &endptr, 16);
     bool failure = (errno || numstr == endptr || *endptr != '\0');
-    free(numstr);
+    g_free(numstr);
     if (failure) /* number too big or some other error */
         return 0;
 
@@ -694,53 +568,53 @@ sr_bin2hex(char *dst, const char *str, int count)
 char *
 sr_indent(const char *input, int spaces)
 {
-    struct sr_strbuf *strbuf = sr_strbuf_new();
+    GString *strbuf = g_string_new(NULL);
     if (*input)
     {
         for (int i = 0; i < spaces; ++i)
-            sr_strbuf_append_char(strbuf, ' ');
+            g_string_append_c(strbuf, ' ');
     }
 
     char *indented = sr_indent_except_first_line(input, spaces);
-    sr_strbuf_append_str(strbuf, indented);
-    free(indented);
+    g_string_append(strbuf, indented);
+    g_free(indented);
 
-    return sr_strbuf_free_nobuf(strbuf);
+    return g_string_free(strbuf, FALSE);
 }
 
 char *
 sr_indent_except_first_line(const char *input, int spaces)
 {
-    struct sr_strbuf *strbuf = sr_strbuf_new();
+    GString *strbuf = g_string_new(NULL);
 
     const char *c = input;
     while (*c)
     {
         if (*c == '\n')
         {
-            sr_strbuf_append_char(strbuf, '\n');
+            g_string_append_c(strbuf, '\n');
             if (*++c)
             {
                 for (int i = 0; i < spaces; ++i)
-                    sr_strbuf_append_char(strbuf, ' ');
+                    g_string_append_c(strbuf, ' ');
             }
 
             continue;
         }
         else
-            sr_strbuf_append_char(strbuf, *c);
+            g_string_append_c(strbuf, *c);
 
         ++c;
     }
 
-    return sr_strbuf_free_nobuf(strbuf);
+    return g_string_free(strbuf, FALSE);
 }
 
 char *
 sr_build_path(const char *first_element, ...)
 {
-    struct sr_strbuf *strbuf = sr_strbuf_new();
-    sr_strbuf_append_str(strbuf, first_element);
+    GString *strbuf = g_string_new(NULL);
+    g_string_append(strbuf, first_element);
 
     va_list elements;
     va_start(elements, first_element);
@@ -748,12 +622,12 @@ sr_build_path(const char *first_element, ...)
     const char *element;
     while ((element = va_arg(elements, const char *)))
     {
-        sr_strbuf_append_char(strbuf, '/');
-        sr_strbuf_append_str(strbuf, element);
+        g_string_append_c(strbuf, '/');
+        g_string_append(strbuf, element);
     }
 
     va_end(elements);
-    return sr_strbuf_free_nobuf(strbuf);
+    return g_string_free(strbuf, FALSE);
 }
 
 static void
@@ -812,8 +686,8 @@ sr_parse_os_release(const char *input, void (*callback)(char*, char*, void*),
             goto skip_line;
         }
 
-        char *key = sr_strndup(cursor, key_end - cursor);
-        char *value = sr_strndup(key_end + 1, value_end - key_end - 1);
+        char *key = g_strndup(cursor, key_end - cursor);
+        char *value = g_strndup(key_end + 1, value_end - key_end - 1);
         unescape_osinfo_value(value, value);
 
         warn("os-release:%u: parsed line: '%s'='%s'", line, key, value);
@@ -865,8 +739,8 @@ anonymize_path(char *orig_path)
         if (new_path)
         {
             // Join /home/anonymized/ and ^
-            new_path = sr_asprintf("%s%s", ANONYMIZED_PATH, new_path);
-            free(orig_path);
+            new_path = g_strdup_printf("%s%s", ANONYMIZED_PATH, new_path);
+            g_free(orig_path);
             return new_path;
         }
     }
