@@ -5,7 +5,6 @@
 #include "py_gdb_sharedlib.h"
 #include "py_base_thread.h"
 #include "py_base_stacktrace.h"
-#include "strbuf.h"
 #include "gdb/stacktrace.h"
 #include "gdb/thread.h"
 #include "gdb/frame.h"
@@ -96,7 +95,7 @@ PyTypeObject sr_py_gdb_stacktrace_type = {
     sizeof(struct sr_py_gdb_stacktrace),        /* tp_basicsize */
     0,                              /* tp_itemsize */
     sr_py_gdb_stacktrace_free,       /* tp_dealloc */
-    NULL,                           /* tp_print */
+    0,                              /* tp_vectorcall_offset */
     NULL,                           /* tp_getattr */
     NULL,                           /* tp_setattr */
     NULL,                           /* tp_compare */
@@ -340,12 +339,12 @@ PyObject *
 sr_py_gdb_stacktrace_str(PyObject *self)
 {
     struct sr_py_gdb_stacktrace *this = (struct sr_py_gdb_stacktrace *)self;
-    struct sr_strbuf *buf = sr_strbuf_new();
-    sr_strbuf_append_strf(buf, "Stacktrace with %zd threads",
+    GString *buf = g_string_new(NULL);
+    g_string_append_printf(buf, "Stacktrace with %zd threads",
                            (ssize_t)(PyList_Size(this->threads)));
-    char *str = sr_strbuf_free_nobuf(buf);
+    char *str = g_string_free(buf, FALSE);
     PyObject *result = Py_BuildValue("s", str);
-    free(str);
+    g_free(str);
     return result;
 }
 
@@ -533,7 +532,10 @@ sr_py_gdb_stacktrace_normalize(PyObject *self, PyObject *args)
     Py_DECREF(this->threads);
 
     this->stacktrace->threads = tmp->threads;
+    this->stacktrace->crash = tmp->crash;
+    this->stacktrace->crash_tid = tmp->crash_tid;
     tmp->threads = NULL;
+    tmp->crash = NULL;
     sr_gdb_stacktrace_free(tmp);
 
     this->threads = thread_linked_list_to_python_list(this->stacktrace);
@@ -567,6 +569,6 @@ sr_py_gdb_stacktrace_to_short_text(PyObject *self, PyObject *args)
 
     PyObject *result = PyString_FromString(text);
 
-    free(text);
+    g_free(text);
     return result;
 }
