@@ -1,8 +1,9 @@
 #include "py_metrics.h"
 #include "py_base_thread.h"
 #include "py_common.h"
-#include "strbuf.h"
+#include "utils.h"
 #include "distance.h"
+#include <glib.h>
 
 #define distances_doc "satyr.Distances - class representing distances between objects\n\n" \
                       "Usage:\n\n" \
@@ -78,7 +79,7 @@ sr_py_distances_type =
     sizeof(struct sr_py_distances),    /* tp_basicsize */
     0,                          /* tp_itemsize */
     sr_py_distances_free,       /* tp_dealloc */
-    NULL,                       /* tp_print */
+    0,                          /* tp_vectorcall_offset */
     NULL,                       /* tp_getattr */
     NULL,                       /* tp_setattr */
     NULL,                       /* tp_compare */
@@ -128,7 +129,7 @@ sr_py_distances_part_type =
     sizeof(struct sr_py_distances_part), /* tp_basicsize */
     0,                          /* tp_itemsize */
     sr_py_distances_part_free,  /* tp_dealloc */
-    NULL,                       /* tp_print */
+    0,                          /* tp_vectorcall_offset */
     NULL,                       /* tp_getattr */
     NULL,                       /* tp_setattr */
     NULL,                       /* tp_compare */
@@ -277,12 +278,12 @@ PyObject *
 sr_py_distances_str(PyObject *self)
 {
     struct sr_py_distances *this = (struct sr_py_distances *)self;
-    struct sr_strbuf *buf = sr_strbuf_new();
-    sr_strbuf_append_strf(buf, "%d-by-%d distance matrix",
+    GString *buf = g_string_new(NULL);
+    g_string_append_printf(buf, "%d-by-%d distance matrix",
                            this->distances->m, this->distances->n);
-    char *str = sr_strbuf_free_nobuf(buf);
+    char *str = g_string_free(buf, FALSE);
     PyObject *result = Py_BuildValue("s", str);
-    free(str);
+    g_free(str);
     return result;
 }
 
@@ -438,7 +439,7 @@ sr_py_distances_part_new(PyTypeObject *object, PyObject *args, PyObject *kwds)
 
     if (PyList_Check(dist_list))
     {
-        part->distances = sr_malloc_array(sizeof(float), part->len);
+        part->distances = g_malloc_n(sizeof(float), part->len);
         int i;
         for (i = 0; i < PyList_Size(dist_list); i++)
         {
